import 'package:price_module/eventbus/notify_event_type.dart';
import 'package:price_module/eventbus/platform_icon_event.dart';
import 'package:price_module/framework/action/ds_action.dart';
import 'package:price_module/framework/model/ds_model.dart';
import 'package:price_module/page/model/details/index.dart';
import 'package:price_module/page/service/details/index.dart';
import 'package:price_module/utils/event_bus_utils.dart';
import 'package:price_module/utils/time_utils.dart';
import 'package:xiaoxiong_repository/app_xiaoxiong_repository.dart';
import 'package:xiaoxiong_repository/entity/turn_chain_entity.dart';
import 'package:price_module/framework/exception/ds_net_exception.dart';

abstract class IPriceDetailsAction<PriceDetailsVo>
    extends IDSAction<PriceDetailsVo> {
  ///
  /// 商品举报
  /// [reportType]              举报类型
  ///
  void report({String? reportType});

  ///
  /// 用户点赞
  /// [status]              点赞的状态
  ///
  void userLike({bool status = true});

  ///
  /// 用户收藏
  /// [status]              收藏的状态
  ///
  void userCollect({bool status = true});

  ///
  /// 用户点击购买
  ///
  Future<TurnChainEntity?> goToBuyEvent();

  ///
  /// 凑单购买
  /// [itemId]              商品Id
  /// [platform]            平台
  ///
  ///
  Future<TurnChainEntity?> goToBuyItemId({String? itemId, String? platform});

  ///
  /// 获取举报配置信息
  ///
  List<dynamic> get sellReportList;
}

class PriceDetailsAction extends IPriceDetailsAction<PriceDetailsVo> {
  PriceDetailsService _detailsService = PriceDetailsService();

  IPriceDetailsActionDelegate? _actionDelegate;

  Map<int, dynamic>? _platformIconMap;
  List<dynamic> _sellReportList = [];

  @override
  List<dynamic> get sellReportList => _sellReportList;

  PriceDetailsAction(
      {required Map<String, dynamic> param, IPriceDetailsActionDelegate? delegate}) {
    eventBus.on<PlatformIconEvent>().listen((event) {
      _platformIconMap = event.map;
    });
    eventBus.fire(NotifyEventType(eventType: NotifyEventEnum.PlatformIconEnum));

    this._actionDelegate = delegate;
    this.vm = PriceDetailsVo(productId: param['productId']);
    if (param['src'] != null && param['src'] is String) {
      this.vm!.itemPic = [param['src']];
    }
    if (param['title'] != null && param['title'] is String) {
      this.vm!.itemTitle = param['title'];
    }
    if (param['sellReportList'] is List) {
      _sellReportList = List<dynamic>.from(param['sellReportList']);
      try {
        this.vm!.reports =
            List<ReportModel>.generate(sellReportList.length, (index) {
          Map<String, dynamic> sellReport =
              Map<String, dynamic>.from(sellReportList[index]);
          List<dynamic> subList = List<dynamic>.from(sellReport['sub']);
          return ReportModel(
              text: sellReport['text'],
              value: sellReport['value'],
              sub: List<ReportModel>.generate(subList.length, (index) {
                Map<String, dynamic> sellSUb =
                    Map<String, dynamic>.from(subList[index]);
                return ReportModel(
                    sub: [], text: sellSUb['text'], value: sellSUb['value']);
              }));
        });
      } catch (e) {
        print(e);
      }
    }
  }

  @override
  void report({String? reportType}) async {
    _detailsService
        .report(id: this.vm!.productId!, reportType: reportType!)
        .asStream()
        .first
        .then((value) {
      if (value['code'] == 2000) {
        _actionDelegate?.showTipsMsg(msg: '举报成功,感谢您的反馈!');
      }
    });
  }

  bool disableLike = false, disableCollect = false;

  @override
  void userLike({bool status = true}) {
    if (disableLike) {
      return;
    }
    disableLike = true;
    _detailsService
        .userLike(
            markId: this.vm!.productId!,
            platform: this.vm!.platform!,
            status: status ? '2' : '1')
        .asStream()
        .first
        .then((value) {
          if (value['code'] == 2000) {
            if (status) {
              try {
                this.vm!.likeTimes = '${int.parse(this.vm!.likeTimes!) - 1}';
              } catch (e) {}
            } else {
              try {
                this.vm!.likeTimes = '${int.parse(this.vm!.likeTimes!) + 1}';
              } catch (e) {}
            }
            this.vm!.isLike = !status;
            this.vm!.notifyListener(true);
          }
        })
        .catchError((error) {})
        .whenComplete(() {
          disableLike = false;
        });
  }

  @override
  void userCollect({bool status = true}) {
    if (disableCollect) {
      return;
    }
    disableCollect = true;
    _detailsService
        .userCollect(
            itemId: this.vm!.productId!,
            platform: this.vm!.platform!,
            status: status ? '2' : '1')
        .asStream()
        .first
        .then((value) {
          if (value['code'] == 2000) {
            if (!status) {
              _actionDelegate?.showTipsMsg(msg: '收藏成功');
            }
            if (status) {
              try {
                this.vm!.collegeTimes = '${int.parse(this.vm!.collegeTimes!) - 1}';
              } catch (e) {}
            } else {
              try {
                this.vm!.collegeTimes = '${int.parse(this.vm!.collegeTimes!) + 1}';
              } catch (e) {}
            }
            this.vm!.isFavorites = !status;
            this.vm!.notifyListener(true);
          }
        })
        .catchError((error) {})
        .whenComplete(() {
          disableCollect = false;
        });
  }

  @override
  void initState() async {
    this.vm!.viewState = DSViewState.busy;
    _detailsService
        .qryProductDetails(productId: this.vm!.productId!)
        .asStream()
        .first
        .then((value) {
      this.vm!.viewState = DSViewState.idle;
      this.vm!.itemPic = value.itemPic;
      this.vm!.itemId = value.itemId;
      this.vm!.platform = value.platform;
      this.vm!.platformText = _getPlatformName(int.parse(value.platform!));
      this.vm!.itemTitle = value.itemTitle;
      this.vm!.subTitle = value.subTitle;
      this.vm!.status = value.status;
      this.vm!.likeTimes = value.likeTimes;
      this.vm!.collegeTimes = value.collegeTimes;
      this.vm!.buttonText = value.buttonText;
      this.vm!.description = value.description;
      this.vm!.releaseTime = value.releaseTime;
      this.vm!.couponText = value.couponText;
      this.vm!.checkText = value.checkText;
      this.vm!.coupons = value.coupons;
      this.vm!.shareUrl = value.shareUrl;
      this.vm!.price = value.price;
      this.vm!.endPrice = value.endPrice;
      this.vm!.shareMessage = value.shareMessage;
      this.vm!.isLike = value.isLike;
      this.vm!.isFavorites = value.isFavorites;
      this.vm!.isOverdue = value.isOverdue;
      this.vm!.shopTitle = value.shopTitle;
      this.vm!.rebate = value.rebate;
      this.vm!.collectOrders = value.collectOrders;
      this.vm!.newItemInfo = value.newItemInfo;
    }).catchError((error) {
      this.vm!.viewState = DSViewState.idle;
    }).catchError((error) {
      this.vm!.viewState = DSViewState.error;
    }).whenComplete(() {
      _detailsService
          .getSellingRecommends(
        excludeId: this.vm!.productId,
        keywords: this.vm!.itemTitle,
        page: 1,
        pagesize: 7,
      )
          .then((value) {
        if (value.isSuccess) {
          this.vm!.productEntitys = value.data;
          this.vm!.productEntitys!.forEach((ele) {
            ele.releaseTime = TimeUtils.formatTime(timestamp: ele.releaseTime!);
            ele.platform = _getPlatformName(int.parse(ele.platform!));
          });
        }
      }).catchError((error) {
        print(error);
      }).whenComplete(() {
        this.vm!.notifyListener(true);
      });
    });
  }

  ///
  /// 将平台的id,转换为平台的名称
  /// [platform]              平台id
  ///
  String _getPlatformName(int platform) {
    if (_platformIconMap != null && _platformIconMap![platform] is Map) {
      Map<String, dynamic> platformMap =
          Map<String, dynamic>.from(_platformIconMap![platform]);
      return platformMap['name'] ?? '';
    }
    return '';
  }

  @override
  Future<TurnChainEntity?> goToBuyEvent() async {
    DSResponseObject<TurnChainEntity> resObject = await _detailsService
        .turnChainForItemId(
            itemId: this.vm!.itemId!, platform: this.vm!.platform!, type: '6')
        .asStream()
        .first;
    if (resObject.isSuccess) {
      return resObject.data;
    }
    throw DSNetException(code: resObject.code, message: resObject.msg);
  }

  @override
  Future<TurnChainEntity?> goToBuyItemId({
    String? itemId,
    String? platform,
  }) async {
    if (platform == null) {
      platform = this.vm!.platform;
    }
    DSResponseObject<TurnChainEntity> resObject = await _detailsService
        .turnChainForItemId(itemId: itemId!, platform: platform!, type: '6')
        .asStream()
        .first;
    if (resObject.isSuccess) {
      return resObject.data;
    }
    throw DSNetException(code: resObject.code, message: resObject.msg);
  }
}

///
/// 代理类，类似接口
///
abstract class IPriceDetailsActionDelegate {
  void showTipsMsg({required String msg});
}
