import 'package:price_module/eventbus/notify_event_type.dart';
import 'package:price_module/eventbus/platform_icon_event.dart';
import 'package:price_module/framework/action/ds_action.dart';
import 'package:price_module/framework/model/ds_model.dart';
import 'package:price_module/page/model/discount/index.dart';
import 'package:price_module/utils/event_bus_utils.dart';
import 'package:price_module/utils/time_utils.dart';
import 'package:xiaoxiong_repository/entity/product_item_entity.dart';
import 'package:xiaoxiong_repository/repository/price_repository.dart';

abstract class IPriceDiscountAction<PriceDiscountVo>
    extends IDSAction<PriceDiscountVo> {
  IPriceDiscountAction();

  ///
  /// 加载数据
  ///
  void reloadData();

  ///
  /// 加载更多
  ///
  void onLoad();

  ///
  /// 获取当前数据
  ///
  PriceDiscountVo get getVo;
}

class PriceDiscountAction extends IPriceDiscountAction<PriceDiscountVo?> {
  PriceRepository _priceRepository = PriceRepository.get();
  Map<int, dynamic>? _platformIconMap;
  IPriceDiscountDelegate? _discountDelegate;
  Map<String, dynamic>? _pageParam;

  PriceDiscountAction({
    IPriceDiscountDelegate? delegate,
    Map<String, dynamic>? pageParam,
  }) {
    this._pageParam = pageParam;
    _discountDelegate = delegate;
    this.vm = PriceDiscountVo();
    eventBus.on<PlatformIconEvent>().listen((event) {
      _platformIconMap = event.map;
    });
    eventBus.fire(NotifyEventType(eventType: NotifyEventEnum.PlatformIconEnum));
  }

  @override
  void initState() {
    this._getSellingRecommends(pagesize: 10, page: 1, isFirst: true);
  }

  int currentPage = 1;

  @override
  void reloadData() {
    this._getSellingRecommends(
      page: 1,
      pagesize: 10,
      isRefresh: true,
    );
    this.vm!.notifyListener();
  }

  @override
  void onLoad() {
    this._getSellingRecommends(
      page: currentPage + 1,
      pagesize: 10,
      isLoad: true,
    );
  }

  ///
  /// 查询数据
  ///
  void _getSellingRecommends({
    int page = 1,
    int pagesize = 10,
    bool isFirst = false,
    bool isLoad = false,
    bool isRefresh = false,
  }) {
    currentPage = page;
    if (isFirst) {
      this.vm!.viewState = DSViewState.busy;
    }
    _priceRepository
        .getSellingRecommends(
      keywords: _pageParam!['keywords'],
      excludeId: _pageParam!['excludeId'],
      page: page,
      pagesize: pagesize,
    )
        .then((value) {
      if (value.isSuccess) {
        List<ProductItemEntity> productEntitys = value.data;

        productEntitys.forEach((ele) {
          ele.releaseTime = TimeUtils.formatTime(timestamp: ele.releaseTime!);
          ele.platform = _getPlatformName(int.parse(ele.platform!));
        });
        if (isLoad) {
          if (this.vm!.productEntitys == null) {
            this.vm!.productEntitys = List<ProductItemEntity>.empty(growable: true);
          }
          this.vm!.productEntitys!.addAll(productEntitys);
        } else {
          this.vm!.productEntitys = productEntitys;
          if (this.vm!.productEntitys!.isEmpty) {
            this.vm!.viewState = DSViewState.empty;
          } else {
            this.vm!.viewState = DSViewState.idle;
          }
        }
      }
      if (isLoad) {
        _discountDelegate!.onLoadComplete(isLoading: true);
      } else if (isRefresh) {
        _discountDelegate!.onLoadComplete(isLoading: false);
      }
    }).catchError((error) {
      this.vm!.viewState = DSViewState.error;
      if (isLoad) {
        _discountDelegate!.onloadFailed(isLoading: true);
      } else if (isRefresh) {
        _discountDelegate!.onloadFailed(isLoading: false);
      }
    }).whenComplete(() {
      this.vm!.notifyListener();
    });
  }

  ///
  /// 将平台的id,转换为平台的名称
  /// [platform]              平台id
  ///
  String _getPlatformName(int platform) {
    if (_platformIconMap != null && _platformIconMap![platform] is Map) {
      Map<String, dynamic> platformMap =
          Map<String, dynamic>.from(_platformIconMap![platform]);
      return platformMap['name'] ?? '';
    }
    return '';
  }

  @override
  PriceDiscountVo? get getVo => this.vm;
}

///
/// 代理类，类似接口
///
abstract class IPriceDiscountDelegate {
  ///
  /// 数据加载完成
  ///
  void onLoadComplete({bool isLoading = true});

  ///
  /// 数据加载失败
  ///
  void onloadFailed({bool isLoading = true, String? msg});
}
