import 'package:price_module/framework/action/ds_action.dart';
import 'package:price_module/framework/model/ds_model.dart';
import 'package:price_module/page/model/index.dart';
import 'package:price_module/utils/time_utils.dart';
import 'package:xiaoxiong_repository/entity/product_item_entity.dart';
import 'package:xiaoxiong_repository/repository/price_repository.dart';

abstract class IPriceAction<PriceVo> extends IDSAction<PriceVo> {
  ///
  /// 代理设置
  ///
  IPriceAction({IPriceActionDelegate? delegate, bool? isGrid});

  ///
  /// 设置分类参数
  ///
  void setPriceCategory(
      {List<dynamic>? sellingTabList,
      List<dynamic>? priceCategory,
      Map<int, dynamic>? platformIconMap,
      String? keywords});

  ///
  /// 获取分类Tab
  ///
  TabListVo getTabListVo();

  ///
  /// 获取快捷分类列表
  ///
  List<dynamic> get sellingTabs;

  ///
  /// 获取产品列表
  ///
  ProductListVo getProductListVo();

  ///
  /// 关闭Tab
  ///
  void closeTab();

  ///
  /// 重置筛选条件
  ///
  void resetProductList();

  ///
  /// 查询筛选条件
  ///
  void qryProductListEvent({String? keywords, int? page});

  ///
  /// 刷新
  ///
  void onRefresh();

  ///
  /// 加载更多
  ///
  void loadMore();
}

class PriceAction extends IPriceAction<PriceVo> {
  PriceRepository _repository = PriceRepository.get();
  Map<int, dynamic>? platformIconMap;
  String? keywords;
  bool? isGrid = false;
  IPriceActionDelegate? delegate;
  PriceAction({this.delegate, this.isGrid}) : super(delegate: delegate);

  @override
  void initState() async {
    if (this.vm == null) {
      this.vm = PriceVo();
    }
    if (this.vm?.productListVo == null) {
      this.vm?.productListVo = ProductListVo();
    }
    this.vm?.productListVo?.isGrid = isGrid;
    this.vm?.productListVo?.viewState = DSViewState.busy;
  }

  @override
  List<dynamic> get sellingTabs => this.vm!.sellingTabs ?? [];

  @override
  void setPriceCategory(
      {List<dynamic>? sellingTabList,
      List<dynamic>? priceCategory,
      Map<int, dynamic>? platformIconMap,
      String? keywords}) {
    this.platformIconMap = platformIconMap;
    this.page = 1;
    if (keywords != null && keywords.isNotEmpty) {
      isGrid = true;
    }
    this.keywords = keywords;
    if (this.vm == null) {
      this.vm = PriceVo();
    }
    if (this.vm!.tabListVo == null) {
      this.vm!.tabListVo = TabListVo();
    }
    this.vm!.sellingTabs = sellingTabList;
    dynamic data = this
        .vm!
        .sellingTabs
        ?.singleWhere((ele) => ele['is_default'] == '1', orElse: () => null);
    if (data != null && data is Map<String, dynamic>) {
      this.vm!.tabListVo!.sellTab = Map<String, dynamic>.from(data);
    }
    if (priceCategory != null) {
      List<TabVo> tabList = List<TabVo>.empty(growable: true);
      priceCategory.forEach((ele) {
        Map<String, dynamic> categoryMap = Map<String, dynamic>.from(ele);
        tabList.add(TabVo.fromJson(
            '${categoryMap['type']}', Map<String, dynamic>.from(ele)));
      });
      TabListVo tabListVo = this.vm!.tabListVo!;
      tabListVo.tabList = tabList;
      this.vm!.tabListVo = tabListVo;
    }

    TabListVo tabListVo = this.vm!.tabListVo!;
    String? categoryId;
    if (tabListVo.sellTab != null && tabListVo.sellTab!['id'] != null) {
      categoryId = "${tabListVo.sellTab!['id']}";
    }
    _qryProductList(keywords: keywords, categoryId: categoryId);
  }

  @override
  TabListVo getTabListVo() {
    return this.vm!.tabListVo ?? TabListVo();
  }

  @override
  ProductListVo getProductListVo() {
    return this.vm?.productListVo ?? ProductListVo(productList: []);
  }

  ///
  /// 重置筛选条件
  ///
  @override
  void resetProductList() async {
    for (TabVo item in this.vm!.tabListVo?.tabList ?? []) {
      item.selected = null;
      item.open = false;
    }
    this.vm!.tabListVo?.sellTab = null;
    this.vm!.tabListVo?.notifyListener(true);
    _qryProductList(keywords: this.keywords);
  }

  ///
  /// 过滤筛选条件
  ///
  @override
  void qryProductListEvent({String? keywords, int? page = 1}) async {
    if (keywords != null) {
      this.keywords = keywords;
    }
    List<dynamic>? filters;
    this.vm!.tabListVo?.tabList.forEach((ele) {
      if (filters == null) {
        filters = List<dynamic>.empty(growable: true);
      }
      if (ele.selected != null) {
        Map<String, dynamic> data;
        if (ele.keyName == 'category') {
          data = {
            'key_name': ele.keyName,
            'key_value': '${ele.selected!['id']}'
          };
        } else if (ele.keyName == 'price_section') {
          data = {
            'key_name': ele.keyName,
            'key_value': ele.selected!['price_section']
          };
        } else {
          data = {
            'key_name': ele.keyName,
            'key_value': '${ele.selected!['type']}'
          };
        }
        if (data.isNotEmpty) {
          filters!.add(data);
        }
      }
    });
    TabListVo tabListVo = this.vm!.tabListVo!;
    String? categoryId;
    if (tabListVo.sellTab != null && tabListVo.sellTab!['id'] != null) {
      categoryId = "${tabListVo.sellTab!['id']}";
    }
    _qryProductList(
        keywords: this.keywords,
        filters: filters,
        page: page!,
        categoryId: categoryId);
  }

  @override
  void closeTab() {
    TabListVo tabListVo = this.vm!.tabListVo!;
    tabListVo.tabList.forEach((ele) {
      ele.open = false;
    });
    tabListVo.notifyListener(true);
  }

  String getPlatformName(int platform) {
    if (platformIconMap != null && platformIconMap![platform] is Map) {
      Map<String, dynamic> platformMap =
          Map<String, dynamic>.from(platformIconMap![platform]);
      return platformMap['name'] ?? '';
    }
    return '';
  }

  void _qryProductList(
      {int page = 1,
      int pagesize = 10,
      String? keywords,
      String? categoryId,
      List<dynamic>? filters}) async {
    this.page = page;
    await _repository
        .qryProductList(
      page: page,
      pagesize: pagesize,
      keywords: keywords,
      categoryId: categoryId,
      filters: filters,
    )
        .then((value) {
      this.vm?.productListVo?.viewState = DSViewState.idle;
      if (page == 1) {
        List<ProductItemEntity> productList = [];
        if (value.isSuccess) {
          productList = value.data;
          productList.forEach((ele) {
            ele.releaseTime = TimeUtils.formatTime(timestamp: ele.releaseTime!);
            ele.platform = getPlatformName(int.parse(ele.platform!));
          });
        }
        this.vm?.productListVo?.productList = productList;
      } else {
        List<ProductItemEntity> productList = [];
        if (value.isSuccess) {
          productList = value.data;
          productList.forEach((ele) {
            ele.releaseTime = TimeUtils.formatTime(timestamp: ele.releaseTime!);
            ele.platform = getPlatformName(int.parse(ele.platform!));
          });
          this.vm?.productListVo?.productList.addAll(productList);
        }
      }
      delegate?.onLoadComplete();
    }).catchError((error) {
      this.vm?.productListVo?.viewState = DSViewState.error;
      delegate?.onloadFailed();
    }).whenComplete(() {
      this.vm?.productListVo?.notifyListener(true);
    });
  }

  int page = 1;

  @override
  void loadMore() {
    qryProductListEvent(page: page + 1);
  }

  @override
  void onRefresh() {
    qryProductListEvent(page: 1);
  }
}

///
/// 代理类，类似接口
///
abstract class IPriceActionDelegate {
  ///
  /// 数据加载完成
  ///
  void onLoadComplete();

  ///
  /// 数据加载失败
  ///
  void onloadFailed({String? msg});
}
