import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_boost/flutter_boost.dart';
import 'package:flutter_html/flutter_html.dart';
import 'package:flutter_native_toast/flutter_native_toast.dart';
import 'package:fluwx/fluwx.dart';
import 'package:price_module/framework/base/ds_provider_widget.dart';
import 'package:price_module/framework/exception/ds_net_exception.dart';
import 'package:price_module/framework/model/ds_model.dart';
import 'package:price_module/page/action/details/index.dart';
import 'package:price_module/page/model/details/index.dart';
import 'package:price_module/page/widget/details/index.dart';
import 'package:price_module/route/index.dart';
import 'package:common_module/utils/xapp_utils.dart';
import 'package:xiaoxiong_repository/entity/product_item_entity.dart';

class PriceDetailsPage extends StatefulWidget {
  final Map<String, dynamic> pageParam;
  final Function(Map<String, dynamic>)? onCallbackParams;
  final Function(String?, String?)? onH5Callback;
  final Function(String?) onCopyLink;
  final Widget Function(
    BuildContext context,
    PriceDetailsVo detailsVo,
    GlobalKey convertKey,
  )? onShareWidget;
  final Function(BuildContext context,
      {String? path, String? type, required GlobalKey convertKey}) onShare;
  const PriceDetailsPage(
      {Key? key,
      required this.pageParam,
      this.onShareWidget,
      this.onCallbackParams,
      this.onH5Callback,
      required this.onCopyLink,
      required this.onShare})
      : super(key: key);

  @override
  _PriceDetailsPageState createState() => _PriceDetailsPageState();
}

class _PriceDetailsPageState extends State<PriceDetailsPage>
    with TickerProviderStateMixin, IPriceDetailsActionDelegate {
  //分享图片key
  GlobalKey _convertKey = GlobalKey();

  IPriceDetailsAction<PriceDetailsVo>? _detailsAction;

  ScrollController scrollController = ScrollController();
  bool silverCollapsed = false;

  ValueNotifier<int> _appBarListenable = ValueNotifier<int>(0);

  AnimationController? likeAnimationController;
  AnimationController? favoritesAnimationController;
  bool? isAudituser;
  @override
  void initState() {
    likeImages.add('assets/images/like/ic_like_unselected.png');
    for (var i = 0; i < 18; i++) {
      likeImages.add('assets/images/like/$i.png');
    }
    favoritesImages.add('assets/images/collect/ic_favorites_unselected.png');
    for (var i = 0; i < 18; i++) {
      favoritesImages.add('assets/images/collect/$i.png');
    }
    likeAnimationController =
        AnimationController(vsync: this, duration: Duration(milliseconds: 400));
    Animation<double> likeAnimation =
        Tween<double>(begin: 1, end: likeImages.length.toDouble() - 1)
            .animate(likeAnimationController!);
    likeAnimationController!.addListener(() {
      likeValueNotifier.value = likeAnimation.value.toInt();
    });
    favoritesAnimationController =
        AnimationController(vsync: this, duration: Duration(milliseconds: 400));
    Animation<double> favoritesAnimation =
        Tween<double>(begin: 1, end: favoritesImages.length.toDouble() - 1)
            .animate(favoritesAnimationController!);
    favoritesAnimationController!.addListener(() {
      favoritesValueNotifier.value = favoritesAnimation.value.toInt();
    });
    super.initState();
    try {
      if (widget.pageParam["isAudituser"] != null) {
        isAudituser = widget.pageParam["isAudituser"] as bool?;
      }
    } catch (e) {}
    _detailsAction =
        PriceDetailsAction(param: widget.pageParam, delegate: this);
    scrollController.addListener(() {
      if (scrollController.offset >= 300) {
        if (silverCollapsed == false) {
          silverCollapsed = true;
          setState(() {});
        }
      } else {
        if (silverCollapsed == true) {
          silverCollapsed = false;
          setState(() {});
        }
      }
    });
  }

  ///
  /// 好价详情
  ///
  Widget _buildBody(BuildContext context,
      {ScrollController? controller, required PriceDetailsVo detailsVo}) {
    return NotificationListener(
      child: Stack(children: [
        detailsVo.viewState == DSViewState.busy
            ? Container()
            : widget.onShareWidget!(context, detailsVo, _convertKey),
        Container(
            color: Colors.white,
            child: ListView(
              children: [
                Container(
                  padding: EdgeInsets.only(left: 13.w, right: 13.w),
                  child: Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      //顶部的滚动栏
                      buildSwiperWidget(images: detailsVo.itemPic ?? []),

                      SizedBox(
                        height: 16,
                      ),
                      Container(
                        child: LayoutBuilder(builder:
                            (BuildContext context, BoxConstraints constraints) {
                          String value = detailsVo.itemTitle!;
                          return Text(
                            value,
                            style: TextStyle(
                              fontSize: 18.w,
                              fontWeight: FontWeight.w600,
                              color: Color(0xFF333333),
                            ),
                            overflow: TextOverflow.ellipsis,
                            maxLines: 2,
                          );
                        }),
                      ),
                      SizedBox(
                        height: 12,
                      ),
                      detailsVo.isOverdue == '1'?Text('过期 | ${detailsVo.subTitle ?? ''}',textAlign: TextAlign.center,style:TextStyle(color:Color(0xFF999999),fontWeight: FontWeight.w600,fontSize: 18.w,)):Text('${detailsVo.subTitle ?? ''}',style:TextStyle(
                        color: Color(0xFFFF0400),
                        fontWeight: FontWeight.w600,fontSize: 18.w,
                      )),
                      /// 商品最新信息
                      buildProductNewlastWidget(
                          newInfo: detailsVo.newItemInfo,
                          goBuy: () {
                            _detailsAction!.goToBuyEvent().then((value) {
                              if (value != null &&
                                  widget.onH5Callback != null) {
                                widget.onH5Callback!(
                                    value.appLink, value.platform);
                              }
                            }).catchError((error) {
                              if (error is DSNetException) {
                                DSNetException exception = error;
                                FlutterNativeToast.showToast(
                                    '${exception.message}');
                              }
                            });
                          },
                          onCouponTap: (Map<String, dynamic> coupon) {
                            if (coupon['status'] == '1' &&
                                coupon['url'] != null) {
                              if (widget.onH5Callback != null) {
                                widget.onH5Callback!(
                                    coupon['url'], detailsVo.platform);
                              }
                            }
                          }),

                      /// 店铺信息
                      buildStoreInfoWidget(
                        vo: detailsVo,
                        onCouponTap: (Coupon coupon) {
                          if (coupon.status == '1' && coupon.url != null) {
                            if (widget.onH5Callback != null) {
                              widget.onH5Callback!(
                                  coupon.url, detailsVo.platform);
                            }
                          }
                        },
                        onPurchase: (data) {
                          if (data['item_id'] == null) {
                            return;
                          }
                          String? itemId = data['item_id'];
                          String? platform = data['platform'];
                          _detailsAction!
                              .goToBuyItemId(itemId: itemId, platform: platform)
                              .then(
                            (value) {
                              if (value != null &&
                                  widget.onH5Callback != null) {
                                widget.onH5Callback!(
                                    value.appLink, value.platform);
                              }
                            },
                          ).catchError((error) {
                            if (error is DSNetException) {
                              DSNetException exception = error;
                              FlutterNativeToast.showToast(
                                  '${exception.message}');
                            }
                          });
                        },
                      ),
                      //商品详情
                      Container(
                          margin: EdgeInsets.only(top: 16.w),
                          child: Html(data: detailsVo.description ?? '')),
                      //举报和提示
                      detailsVo.viewState == DSViewState.busy
                          ? Container(
                              alignment: Alignment.topCenter,
                              child: CupertinoActivityIndicator(
                                animating: true,
                                radius: 16,
                              ),
                            )
                          : buildPromptWidget(
                              releaseTime: detailsVo.releaseTime,
                              checkText: detailsVo.checkText,
                              extras: {
                                'platform': detailsVo.platform,
                                'item_id': detailsVo.itemId,
                                'product_id': detailsVo.productId
                              },
                              onReportTap: () {
                                _showReceiveModalBottom(context,
                                    reports: detailsVo.reports ?? []);
                              }),
                    ],
                  ),
                ),

                SizedBox(
                  height: 4.w,
                ),

                //相关优惠信息
                buildRelatedDiscountWidget(
                    productEntitys: detailsVo.productEntitys,
                    onMoreTap: () async {
                      await BoostNavigator.instance.push(
                        RoutePath.PRICE_DISCOUNT_PAGE.path(),
                        arguments: {
                          'isAudituser': isAudituser,
                          'sellReportList': _detailsAction!.sellReportList,
                          'excludeId': detailsVo.productId,
                          'keywords': detailsVo.itemTitle,
                        },
                      );
                    },
                    onItemTap: (ProductItemEntity entity, int index) async {
                      Map<String, dynamic> params = {
                        'isAudituser': isAudituser,
                        'sellReportList': _detailsAction!.sellReportList,
                        'productId': entity.id,
                        'src': entity.itemPic,
                        'title': entity.itemTitle,
                        'remark': entity.special,
                      };
                      await BoostNavigator.instance.push(
                        RoutePath.PRICE_DETAIL_PAGE.path(),
                        withContainer: false,
                        arguments: params,
                      );
                    }),
              ],
            )),
        buildAppBar(context,
            audituser: isAudituser,
            detailsVo: detailsVo,
            listenable: _appBarListenable,
            onSharePressed:(PriceDetailsVo? detailsVo)=>onSharePressed(detailsVo: detailsVo)
          ),
      ]),
      onNotification: (ScrollNotification notification) {
        if (notification.metrics.outOfRange == true ||
            notification.metrics.axis != Axis.vertical) {
          return false;
        }
        double pixels = notification.metrics.pixels;
        if (pixels < 50 && _appBarListenable.value != 0) {
          _appBarListenable.value = 0;
        } else if (pixels >= 50 && pixels < 200) {
          int alpha = ((pixels - 50) / 150 * 255).toInt();
          if (alpha != _appBarListenable.value) {
            _appBarListenable.value = alpha;
          }
        } else if (pixels >= 200 && _appBarListenable.value != 255) {
          _appBarListenable.value = 255;
        }
        return true;
      },
    );
  }

  void onSharePressed({required PriceDetailsVo? detailsVo}) {
    List<Map<String, dynamic>> items = [
      {
        'key': 'friend',
        'name': 'assets/images/ic_share_friend.png',
        'title': '朋友圈'
      },
      {
        'key': 'wechat',
        'name': 'assets/images/ic_share_wechat.png',
        'title': '微信好友'
      },
      {
        'key': 'copyLink',
        'name': 'assets/images/ic_share_copy.png',
        'title': '复制链接'
      },
      {
        'key': 'product',
        'name': 'assets/images/ic_share_product.png',
        'title': '分享商品'
      }
    ];
    showShareModalBottom<String>(context, items: items, extras: {
      'product_id': detailsVo!.productId,
      'platform': detailsVo.platform,
      'item_id': detailsVo.itemId,
    }).then((value) async {
      if (value == 'product') {
        if (widget.onCallbackParams != null) {
          if (detailsVo.platform == '10' || detailsVo.platform == '1') {
            widget.onCallbackParams!(
                {'itemId': detailsVo.itemId, 'platform': detailsVo.platform});
          } else {
            _showShareQRModalBottom(context: context, detailsVo: detailsVo);
          }
        }
      } else if (value == 'wechat' || value == 'friend') {
        try {
          WeChatShareWebPageModel model = WeChatShareWebPageModel(
              detailsVo.shareUrl ?? '',
              title: detailsVo.subTitle ?? '小熊有好货',
              description: detailsVo.itemTitle ?? '小熊有好货',
              scene: value == 'wechat'
                  ? WeChatScene.SESSION
                  : WeChatScene.TIMELINE,
              thumbnail: WeChatImage.network(detailsVo.itemPic![0]!));
          shareToWeChat(model).then((value) {});
        } catch (e) {
          print("------------------------------> value:$e");
        }
      } else if (value == 'copyLink') {
        if (detailsVo.shareMessage != null &&
            detailsVo.shareMessage!.isNotEmpty) {
          widget.onCopyLink(detailsVo.shareMessage);
          FlutterNativeToast.showToast('复制成功');
        }
      }
    });
  }

  ///
  /// 京东和拼多多分享图片
  /// context             上下文
  /// detailsVo           商品详情
  ///
  void _showShareQRModalBottom(
      {required BuildContext context, required PriceDetailsVo detailsVo}) {
    List<Map<String, dynamic>> items = [
      {
        'key': 'photo_save',
        'name': 'assets/images/download_icon.png',
        'title': '保存图片'
      },
      {
        'key': 'photo_wechat',
        'name': 'assets/images/ic_share_wechat.png',
        'title': '微信好友'
      },
      {
        'key': 'photo_friend',
        'name': 'assets/images/ic_share_friend.png',
        'title': '朋友圈'
      }
    ];
    showShareModalBottom<String>(context, items: items, extras: {
      'product_id': detailsVo.productId,
      'platform': detailsVo.platform,
      'item_id': detailsVo.itemId,
    }).then((value) async {
      widget.onShare(context,
          path: value, type: value, convertKey: _convertKey);
    });
  }

  ///
  /// 举报弹窗
  /// [context]
  /// [reports]           举报弹窗的信息
  ///
  void _showReceiveModalBottom(BuildContext context,
      {required List<ReportModel> reports}) {
    showReceiveModalBottom<ReportModel>(context, items: reports).then((value) {
      if (value != null) {
        if (value.sub != null && value.sub!.length > 0) {
          _showReceiveModalBottom(context, reports: value.sub!);
        } else {
          _detailsAction!.report(reportType: value.value);
        }
      }
    });
  }

  @override
  Widget build(BuildContext context) {
    return DSProviderWidget<PriceDetailsVo,
        IPriceDetailsAction<PriceDetailsVo>>(
      dsAction: _detailsAction!,
      builder: (BuildContext context, child) {
        return Scaffold(
          backgroundColor: Colors.white,
          body: _buildBody(context,controller: scrollController, detailsVo: _detailsAction!.vm!),
          bottomNavigationBar: _buildNavigationBar(detailsVo: _detailsAction!.vm!, detailsAction: _detailsAction),
        );
      },
    );
  }

  ValueNotifier<int> likeValueNotifier = ValueNotifier(0);
  ValueNotifier<int> favoritesValueNotifier = ValueNotifier(0);

  List<String> likeImages = List<String>.empty(growable: true);
  List<String> favoritesImages = List<String>.empty(growable: true);

  ///
  /// 底部按钮区域
  ///
  Widget _buildNavigationBar({
    required PriceDetailsVo detailsVo,
    IPriceDetailsAction<PriceDetailsVo>? detailsAction,
  }) {
    return buildBottomNavWidget(
      like: detailsVo.likeTimes ?? '0',
      favorites: detailsVo.collegeTimes ?? '0',
      isLike: detailsVo.isLike,
      status: detailsVo.status,
      isFavorites: detailsVo.isFavorites,
      text: detailsVo.viewState == DSViewState.busy
          ? '加载中'
          : detailsVo.buttonText ?? '',
      extras: {
        'platform': detailsVo.platform,
        'item_id': detailsVo.itemId,
        'product_id': detailsVo.productId
      },
      likeImages: likeImages,
      likeValueNotifier: likeValueNotifier,
      favoritesImages: favoritesImages,
      favoritesValueNotifier: favoritesValueNotifier,
      onShareTap: () => onSharePressed(detailsVo: detailsVo),
      onBuyTap: () {
        ///
        /// 如果是淘宝、天猫、和拼多多需要提示授权
        ///
        detailsAction!.goToBuyEvent().then((value) {
          if (value != null && widget.onH5Callback != null) {
            widget.onH5Callback!(value.appLink, value.platform);
          }
        }).catchError((error) {
          if (error is DSNetException) {
            DSNetException exception = error;
            FlutterNativeToast.showToast('${exception.message}');
          }
        });
      },
      onFavoritesTap: () {
        detailsAction!.userCollect(status: detailsVo.isFavorites);
        if (detailsVo.isFavorites) {
          favoritesValueNotifier.value = 0;
        } else {
          favoritesAnimationController!.forward(from: 0);
        }
      },
      onLikeTap: () {
        detailsAction!.userLike(status: detailsVo.isLike);
        if (detailsVo.isLike) {
          likeValueNotifier.value = 0;
        } else {
          likeAnimationController!.forward(from: 0);
        }
      },
    );
  }

  @override
  void dispose() {
    likeAnimationController?.dispose();
    favoritesAnimationController?.dispose();
    likeValueNotifier.dispose();
    favoritesValueNotifier.dispose();
    super.dispose();
  }

  @override
  void showTipsMsg({required String msg}) {
    FlutterNativeToast.showToast(msg);
  }
}
