import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_boost/boost_navigator.dart';
import 'package:price_module/framework/base/ds_provider_widget.dart';
import 'package:price_module/framework/model/ds_model.dart';
import 'package:price_module/page/action/discount/index.dart';
import 'package:price_module/page/model/discount/index.dart';
import 'package:price_module/page/widget/discount/index.dart';
import 'package:price_module/route/index.dart';
import 'package:price_module/widget/failed/failed_load_page.dart';
import 'package:price_module/widget/price/price_item.dart';
import 'package:price_module/widget/pull/pull_widget.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'package:xiaoxiong_repository/entity/product_item_entity.dart';
import 'package:common_module/utils/xapp_utils.dart';

///
/// 好价更多优惠页面
///
class PriceDiscountPage extends StatefulWidget {
  final Map<String, dynamic> pageParam;
  const PriceDiscountPage({Key? key, required this.pageParam})
      : super(key: key);
  @override
  _PriceDiscountPageState createState() => _PriceDiscountPageState();
}

class _PriceDiscountPageState extends State<PriceDiscountPage>
    with IPriceDiscountDelegate {
  IPriceDiscountAction? _discountAction;

  bool? _isAudituser = false;
  List<dynamic>? _sellReportList;
  @override
  void initState() {
    _isAudituser = widget.pageParam['isAudituser'];
    _sellReportList = widget.pageParam['sellReportList'];

    _discountAction = PriceDiscountAction(
      delegate: this,
      pageParam: widget.pageParam,
    );
    super.initState();
  }

  RefreshController _refreshController = RefreshController();

  @override
  Widget build(BuildContext context) {
    return DSProviderWidget<PriceDiscountVo,
        IPriceDiscountAction<PriceDiscountVo>>(
      dsAction: _discountAction as IPriceDiscountAction<PriceDiscountVo>,
      builder: (BuildContext context, Widget? child) {
        return Scaffold(
          appBar: buildAppBar() as PreferredSizeWidget,
          body: _buildBody1(vo: _discountAction!.getVo),
        );
      },
    );
  }

  Widget _buildBody1({required PriceDiscountVo vo}) {
    return PullWidget(
      controller: _refreshController,
      child: Container(child: _buildBody(vo: vo)),
      onRefresh: () {
        _discountAction!.reloadData();
      },
      onLoad: () {
        _discountAction!.onLoad();
      },
    );
  }

  Widget _buildBody({required PriceDiscountVo vo}) {
    if (vo.viewState == DSViewState.busy) {
      return LoadingPage(
        padding: EdgeInsets.all(13.w),
        length: 8,
      );
    } else if (vo.viewState == DSViewState.empty ||
        vo.viewState == DSViewState.error) {
      return FailedLoadPage(
        margin: EdgeInsets.only(top: 68.w),
        errorMsg:
            vo.viewState == DSViewState.empty ? '未发现数据，请刷新重试' : '加载失败，请刷新重试',
        onReloadTap: () {
          //重新加载数据
          _discountAction!.reloadData();
        },
      );
    }
    return PriceListWidget(
      padding: EdgeInsets.only(
        bottom: 20.w,
        left: 13.w,
        right: 13.w,
      ),
      shrinkWrap: true,
      physics: NeverScrollableScrollPhysics(),
      list: vo.productEntitys,
      onItemTap: (ProductItemEntity entity, int index) async {
        Map<String, dynamic> params = {
          'isAudituser': _isAudituser,
          'sellReportList': _sellReportList,
          'productId': entity.id,
          'src': entity.itemPic,
          'title': entity.itemTitle,
          'remark': entity.special
        };
        await BoostNavigator.instance.push(
          RoutePath.PRICE_DETAIL_PAGE.path(),
          withContainer: false,
          arguments: params,
        );
      },
    );
  }

  @override
  void onLoadComplete({bool isLoading = true}) {
    if (isLoading) {
      _refreshController.loadComplete();
    } else {
      _refreshController.refreshCompleted();
    }
  }

  @override
  void onloadFailed({bool isLoading = true, String? msg}) {
    if (isLoading) {
      _refreshController.loadFailed();
    } else {
      _refreshController.refreshFailed();
    }
  }
}
