import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:price_module/framework/base/ds_provider.dart';
import 'package:price_module/framework/base/ds_provider_widget.dart';
import 'package:price_module/framework/model/ds_model.dart';
import 'package:price_module/page/action/index.dart';
import 'package:price_module/page/model/index.dart';
import 'package:price_module/page/widget/index.dart';
import 'package:price_module/widget/failed/failed_load_page.dart';
import 'package:price_module/widget/price/price_item.dart';
import 'package:price_module/widget/pull/pull_widget.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'package:common_module/utils/xapp_utils.dart';
import 'package:common_module/base/base_umeng_page_view_item_state.dart';
import 'package:xiaoxiong_repository/entity/product_item_entity.dart';

abstract class PricePageDelegate {
  ///
  /// 路由
  /// [type]            1、白菜专区 2、爆单页面  3、搜索页面  4、商品详情
  /// [data]            当[type]等于4时,[data]为传递的数据
  ///
  void onRouteTap({required String type, Map<String, dynamic>? data});

  ///
  /// 获取好价的快捷Tab
  ///
  List<dynamic> get sellingTabList;
}

class PricePage extends StatefulWidget {
  final PricePageDelegate? pageDelegate;
  final List<dynamic>? priceCategory;
  final Map<int, dynamic>? platformIconMap;
  const PricePage(
      {Key? key, this.pageDelegate, this.platformIconMap, this.priceCategory})
      : super(key: key);
  @override
  _PricePageState createState() => _PricePageState();
}

class _PricePageState extends BaseUmengPageViewItemState<PricePage>
    with IPriceActionDelegate {
  IPriceAction<PriceVo>? _priceAction;

  @override
  void initState() {
    super.initState();
    _priceAction = PriceAction(delegate: this);
    _priceAction!.setPriceCategory(
        sellingTabList: widget.pageDelegate!.sellingTabList,
        priceCategory: widget.priceCategory,
        platformIconMap: widget.platformIconMap);
  }

  @override
  Widget build(BuildContext context) {
    super.build(context);
    return DSProviderWidget<PriceVo, IPriceAction<PriceVo>>(
        dsAction: _priceAction!,
        builder: (BuildContext context, child) {
          return Scaffold(
            backgroundColor: Color(0xFFF5F5F5),
            appBar: buildAppBar(
                onSearchTap: () {
                  widget.pageDelegate?.onRouteTap(type: '3');
                },
                isGrid: _priceAction!.vm?.productListVo?.isGrid ?? false,
                onLayoutTap: (bool isGrid) async {
                  _priceAction!.vm?.productListVo?.isGrid = isGrid;
                  _priceAction!.vm?.notifyListener(true);
                }) as PreferredSizeWidget?,
            body: _buildBody(action: _priceAction!),
          );
        });
  }

  RefreshController _refreshController = RefreshController();
  ScrollController _scrollViewController = ScrollController();
  ValueNotifier<int> scrollIndex = ValueNotifier<int>(0);
  ValueNotifier<double> backColorListenable = ValueNotifier<double>(136.0.w);
  GlobalKey anchorKey = GlobalKey();

  Widget _buildBody({required IPriceAction<PriceVo> action}) {
    return Stack(
      children: [
        // 底部设置一个背景跟随滑动
        backgroundColorWidget(valueListenable: backColorListenable),

        NotificationListener(
          child: PullWidget(
            child: CustomScrollView(
              controller: _scrollViewController,
              cacheExtent: 1000,
              slivers: [
                //白菜专区和爆款专区
                SliverToBoxAdapter(
                  child: buildZoneWidget(
                    onValueTap: (String type) {
                      widget.pageDelegate?.onRouteTap(type: type);
                    },
                  ),
                ),
                //分类标签
                DSProvider.value(
                    vo: action.getTabListVo(),
                    builderWidget: (BuildContext context, TabListVo tabListVo,
                        Widget? child) {
                      return buildCategoryTapWidget(
                          action.getTabListVo().tabList,
                          action.sellingTabs,
                          anchorKey: anchorKey,
                          sellTab: tabListVo.sellTab,
                          onChange: (int index, bool enable) {
                        tabListVo.tabList[index].open = enable;
                        tabListVo.notifyListener(true);
                      }, onChangeSort: (Map<String, dynamic> data) {
                        tabListVo.sellTab = data;
                        tabListVo.notifyListener(true);
                        _priceAction!.qryProductListEvent(page: 1);
                        double offset = _scrollViewController.offset;
                        if (offset > 70.w) {
                          _scrollViewController.animateTo(70.w,
                              duration: Duration(milliseconds: 50),
                              curve: Curves.linear);
                        }
                      }, onNodeChange: (int? index, TabVo tabVo) {
                        tabVo.open = false;
                        if (tabVo.selected != null &&
                            tabVo.selected!['is_default'] == '1') {
                          tabVo.selected = null;
                        }
                        tabListVo.notifyListener(true);
                        _priceAction!.qryProductListEvent(page: 1);
                        double offset = _scrollViewController.offset;
                        if (offset > 70.w) {
                          _scrollViewController.animateTo(70.w,
                              duration: Duration(milliseconds: 50),
                              curve: Curves.linear);
                        }
                      });
                    }),
                //商品列表
                SliverToBoxAdapter(
                    child: DSProvider.value(
                        vo: action.getProductListVo(),
                        builderWidget: (BuildContext context,
                            ProductListVo productListVo, Widget? child) {
                          if (productListVo.productList.isEmpty) {
                            if (productListVo.viewState == DSViewState.busy) {
                              return LoadingPage(
                                padding: EdgeInsets.all(13.w),
                                length: 6,
                              );
                            } else {
                              return buildEmptyProductList(
                                  refresh: productListVo.viewState ==
                                      DSViewState.error,
                                  onResetTap: () {
                                    action.resetProductList();
                                  });
                            }
                          }

                          if (productListVo.isGrid ?? false) {
                            return PriceGridWidget(
                              shrinkWrap: true,
                              isHaoJia: true,
                              padding: EdgeInsets.only(
                                left: 13.w,
                                right: 13.w,
                                bottom: 13.w,
                              ),
                              physics: NeverScrollableScrollPhysics(),
                              list: productListVo.productList,
                              onItemTap: (int index) {
                                ProductItemEntity itemVo =
                                    productListVo.productList[index];
                                Map<String, dynamic> goodDetails = {
                                  'productId': itemVo.id,
                                  'src': itemVo.itemPic,
                                  'title': itemVo.itemTitle,
                                  'remark': itemVo.special
                                };
                                widget.pageDelegate
                                    ?.onRouteTap(type: '4', data: goodDetails);
                              },
                            );
                          } else {
                            return PriceListWidget(
                              shrinkWrap: true,
                              scroll: false,
                              isHaoJia: true,
                              padding: EdgeInsets.only(
                                  left: 13.w, right: 13.w, bottom: 13.w),
                              physics: NeverScrollableScrollPhysics(),
                              list: productListVo.productList,
                              onItemTap: (ProductItemEntity entity, int index) {
                                Map<String, dynamic> goodDetails = {
                                  'productId': entity.id,
                                  'src': entity.itemPic,
                                  'title': entity.itemTitle,
                                  'remark': entity.special
                                };
                                widget.pageDelegate
                                    ?.onRouteTap(type: '4', data: goodDetails);
                              },
                            );
                          }
                        })),
              ],
            ),
            onLoad: () {
              action.loadMore();
            },
            onRefresh: () {
              action.onRefresh();
            },
            controller: _refreshController,
          ),
          onNotification: (ScrollNotification scroll) {
            if (scroll.metrics.axis != Axis.vertical) {
              return false;
            }

            //回到顶部
            if (scroll.metrics.pixels > 500 && scrollIndex.value == 0) {
              scrollIndex.value = 1;
            } else if (scroll.metrics.pixels <= 500 && scrollIndex.value == 1) {
              scrollIndex.value = 0;
            }
            RenderBox renderBox = anchorKey.currentContext!.findRenderObject() as RenderBox;
            var offset =
                renderBox.localToGlobal(Offset(0.0, renderBox.size.height));
            if (scroll.runtimeType == ScrollStartNotification) {
              print(
                  '=================> 滑动开始 ${scroll.metrics.pixels}  ${offset.dx}  ${offset.dy}');
            } else if (scroll.runtimeType == ScrollEndNotification) {
              print(
                  '=================> 滑动结束 ${scroll.metrics.pixels}  ${offset.dx}  ${offset.dy}');
            } else if (scroll.runtimeType == ScrollUpdateNotification) {
              ///背景色 滑动更新
              double value = (offset.dy - 136.w).ceilToDouble();
              if (value != backColorListenable.value) {
                backColorListenable.value = value;
              }
            }
            return true;
          },
        ),

        //回到顶部
        backTopWidget(
            valueListenable: scrollIndex,
            onPressed: () {
              _scrollViewController.animateTo(0,
                  duration: Duration(milliseconds: 300), curve: Curves.linear);
            })
      ],
    );
  }

  @override
  bool get wantKeepAlive => true;

  @override
  void onLoadComplete() {
    _refreshController.loadComplete();
    _refreshController.refreshCompleted();
  }

  @override
  void onloadFailed({String? msg}) {
    _refreshController.loadFailed();
    _refreshController.refreshCompleted(resetFooterState: true);
  }

  @override
  void dispose() {
    super.dispose();
  }

  @override
  String getPageViewName() {
    return 'entry_price_widget';
  }

  @override
  Map<String, String> getPageViewArgs() {
    return {};
  }
}
