import 'dart:ui';
import 'package:flutter/services.dart';
import 'package:html/dom.dart' as dom;
import 'package:cached_network_image/cached_network_image.dart';
import 'package:common_module/utils/audit_mode_utils.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter_html/flutter_html.dart';
import 'package:flutter_swiper/flutter_swiper.dart';
import 'package:price_module/page/model/details/index.dart';
import 'package:price_module/route/index.dart';
import 'package:price_module/widget/animation/image_animation_widget.dart';
import 'package:price_module/widget/image/image_widget.dart';
import 'package:common_module/utils/xapp_utils.dart';
import 'package:price_module/widget/pop/index.dart';
import 'package:price_module/widget/price/price_item.dart';
import 'package:xiaoxiong_repository/entity/product_item_entity.dart';

///
/// 顶部AppBar
/// [context]                 上下文
/// [audituser]               是否是审核模式
/// [title]                   标题
/// [actions]                 action
///
Widget buildAppBar(BuildContext context,
    {required PriceDetailsVo detailsVo,
    bool? audituser = false,
    String title = '',
    required ValueListenable<int> listenable,
    void Function(PriceDetailsVo detailsVo)? onSharePressed}) {
  return ValueListenableBuilder<int>(
    valueListenable: listenable,
    builder: (BuildContext context, int value, Widget? child) {
      return Container(
          height: 48.w + MediaQuery.of(context).padding.top,
          child: AppBar(
            backgroundColor: Colors.white.withAlpha(value.toInt()),
            elevation: 0,
            title: Text(
              value <= 120 ? '' : '好价详情',
              style: TextStyle(fontSize: 17.w, color: Color(0xFF333333)),
            ),
            centerTitle: true,
            leading: IconButton(
              icon: ImageIcon(
                AssetImage('assets/web_view_back_btn.png'),
                size: 26.w,
              ),
              onPressed: () {
                Navigator.maybePop(context);
              },
            ),
            actions: audituser!
                ? []
                : [IconButton(
                    icon: Image.asset('assets/images/ic_btn_share1.png',package: 'price_module',width: 26.w),
                    onPressed: () => onSharePressed!(detailsVo)
                  )],
          ));
    },
  );
}

///
/// Body区域布局
///
///
Widget buildBody(BuildContext context,
    {required ScrollController controller,
    required List<Widget> slivers,
    PointerMoveEventListener? onPointerMove,
    PointerUpEventListener? onPointerUp}) {
  return Listener(
    onPointerMove: onPointerMove,
    onPointerUp: onPointerUp,
    child: Container(
        color: Color(0xFFF5F5F5),
        child: CustomScrollView(
          physics: ClampingScrollPhysics(),
          controller: controller,
          slivers: slivers,
        )),
  );
}

///
/// 好价商品详情滚动栏图片
/// [images]                滚动栏图片
///
Widget buildSwiperWidget({List<String?>? images}) {
  Widget _swiperWidget;
  int length = (images?.length ?? 0);
  if (length <= 0) {
    _swiperWidget = Image.asset(
      'assets/images/placeholder_image.png',
      package: 'price_module',
      width: 200,
    );
  } else if (length == 1) {
    _swiperWidget = _buildCachedImage(image: images![0]!);
  } else {
    _swiperWidget = Swiper(
      itemCount: length,
      loop: false,
      autoplay: false,
      autoplayDelay: 5 * 1000,
      onIndexChanged: (int? index) {},
      pagination: _buildSwiperPlugin(),
      itemBuilder: (BuildContext context, int index) {
        String image = images![index]!;
        return _buildCachedImage(image: image);
      },
    );
  }
  return Container(
    height: 200,
    alignment: Alignment.bottomCenter,
    child: _swiperWidget,
  );
}

Widget _buildCachedImage({required String image}) {
  return CachedNetworkImage(
    imageUrl: image,
    placeholder: (BuildContext c, String s) {
      return Image.asset(
        'assets/images/placeholder_image.png',
        package: 'price_module',
      );
    },
  );
}

dynamic _buildSwiperPlugin() {
  return SwiperCustomPagination(
      builder: (BuildContext context, SwiperPluginConfig? config) {
    return Positioned(
        bottom: 0.w,
        right: 0.w,
        child: Container(
          width: 43.w,
          height: 21.w,
          alignment: Alignment.center,
          decoration: BoxDecoration(
              color: Color(0xFF979797),
              borderRadius: BorderRadius.circular(10.w)),
          child: Text(
            '${config!.activeIndex! + 1}/${config.itemCount}',
            style: TextStyle(
                color: Colors.white,
                fontSize: 12.w,
                fontWeight: FontWeight.w500),
          ),
        ));
  });
}

///
/// 商品顶部的banner
/// [leading]                     返回布局
/// [action]                      右边菜单
/// [title]                       顶部标题
/// [itemCount]                   banner的个数
/// [itemBuilder]                 banner Item布局
///
Widget buildSliverAppBar(
    {double top = 0,
    required Widget leading,
    required Widget action,
    required bool isBusy,
    required String title,
    required double expandedHeight,
    required int itemCount,
    required IndexedWidgetBuilder itemBuilder}) {
  return SliverAppBar(
      elevation: 0.1,
      backgroundColor: isBusy ? Color(0xFFE9E9E9) : Colors.white,
      leading: leading,
      actions: [action],
      expandedHeight: expandedHeight,
      title: Text(
        title,
        style: TextStyle(fontSize: 17.w, fontWeight: FontWeight.w600),
      ),
      floating: false,
      pinned: true,
      snap: false,
      systemOverlayStyle: SystemUiOverlayStyle(statusBarBrightness: Brightness.light),
      flexibleSpace: FlexibleSpaceBar(
          background: Container(
        margin: EdgeInsets.only(top: top),
        height: 200.w,
        child: isBusy
            ? null
            : Swiper(
                key: UniqueKey(),
                itemCount: itemCount,
                loop: true,
                autoplay: true,
                autoplayDelay: 5 * 1000,
                itemBuilder: itemBuilder,
                onIndexChanged: (int? index) {},
                pagination: _buildCustomPagination(),
              ),
      )));
}

///
/// 商品详情部分
///
Widget buildDetailsWidget({required List<Widget> children}) {
  return SliverToBoxAdapter(
    child: Container(
      color: Colors.white,
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: children,
      ),
    ),
  );
}

///
/// 商品详情标题和副标题
/// [title]                   商品标题
/// [subTitle]                商品副标题
/// [expired]                 是否过期
///
Widget buildTitleWidget({
  required String title,
  required String subTitle,
  bool expired = false,
}) {
  return Container(
      margin: EdgeInsets.only(left: 13.w, right: 13.w),
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          Container(
            margin: EdgeInsets.only(top: 13.w, bottom: 8.w),
            child: Text(
              title,
              style: TextStyle(
                  fontSize: 18.w,
                  fontWeight: FontWeight.w600,
                  color: Color(0xFFFF0400)),
              maxLines: 2,
              overflow: TextOverflow.ellipsis,
            ),
          ),
          Text(
            subTitle,
            style: TextStyle(
                fontSize: 15.w,
                fontWeight: FontWeight.w600,
                color: Color(expired ? 0xFFFF0400 : 0xFF333333)),
            maxLines: 2,
            overflow: TextOverflow.ellipsis,
          ),
        ],
      ));
}

///
/// 商品信息
///
Widget buildStoreInfoWidget(
    {required PriceDetailsVo vo,
    void Function(Coupon)? onCouponTap,
    void Function(Map<String, dynamic>)? onPurchase}) {
  if ((vo.shopTitle ?? '').isEmpty) {
    return Container();
  }

  /// 凑单购买
  Widget getCollectOrders({required List<dynamic> orders}) {
    Widget getOrdersItemWidget(dynamic data) {
      return InkWell(
        child: Container(
            margin: EdgeInsets.only(left: 0),
            child: Row(
              children: [
                Container(
                    child: Text(
                      data['item_title'],
                      style: TextStyle(
                        color: Color(0xFFFF0400),
                        fontSize: 12.w,
                      ),
                      maxLines: 1,
                      overflow: TextOverflow.ellipsis,
                    ),
                    constraints: BoxConstraints(maxWidth: 255)),
                Image.asset(
                  'assets/images/ic_arrow_gray.png',
                  package: 'price_module',
                  width: 14.w,
                  height: 10.w,
                  color: Color(0xFFFF0400),
                )
              ],
            ),),
        onTap: () {
          if (onPurchase != null) {
            onPurchase(data);
          }
        },
      );
    }

    return Wrap(
      runSpacing:6.rpx,
      children: List.generate(
        orders.length,
        (index) => getOrdersItemWidget(orders[index]),
      ),
    );
  }

  ///
  /// 优惠卷Widget
  /// [coupon]            优惠卷
  ///
  Widget getCouponWidget({required Coupon coupon}) {
    return InkWell(child:RichText(text: TextSpan(children:[
      WidgetSpan(child:Container(height:16.w,child:Text(coupon.name!,style: TextStyle(color:Color(coupon.status == '1' ? 0xFFFF0400 : 0xFF999999),fontSize: 12.w)),),),
      WidgetSpan(child:Container(
        width: 16.w,
        alignment:Alignment.centerLeft,
        padding: EdgeInsets.only(top:1.w),
        height:16.w,
        child:Image.asset('assets/images/ic_arrow_gray.png',package: 'price_module',width: 14.w,height: 10.w,color: Color(coupon.status == '1' ? 0xFFFF0400 : 0xFF999999))))
    ])),onTap:(){
      if (onCouponTap != null) {
        onCouponTap(coupon);
      }
    },);
  }

  return Container(
      margin: EdgeInsets.only(top: 16.w),
      padding: EdgeInsets.all(12.w),
      decoration: BoxDecoration(
          color: Color(0xFFFAFAFA),
          borderRadius: BorderRadius.all(Radius.circular(8.0))),
      child: Column(
        children: [
          Row(
            children: [
              Text('店铺',
                  style: TextStyle(fontSize: 12.w, color: Color(0xFF999999))),
              SizedBox(
                width: 12.w,
              ),
              Text(vo.platformText ?? '',
                  style: TextStyle(fontSize: 12.w, color: Color(0xFF302600))),
              Container(
                margin: EdgeInsets.symmetric(horizontal: 8.w),
                color: Color(0xFFD8D8D8),
                width: 0.5.w,
                height: 13.w,
              ),
              Text(vo.shopTitle!,
                  style: TextStyle(fontSize: 12.w, color: Color(0xFF302600))),
            ],
          ),

          //凑单购买
          (vo.collectOrders?.length ?? 0) > 0
              ? Container(
                  child: Row(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      Text('凑单',
                          style: TextStyle(
                              fontSize: 12.w, color: Color(0xFF999999))),
                      SizedBox(
                        width: 12.w,
                      ),
                      Container(
                        width: 280.w,
                        child: getCollectOrders(orders: vo.collectOrders!),
                      ),
                    ],
                  ),
                  margin: EdgeInsets.only(
                    top: 12.w,
                  ),
                )
              : Container(),

          //优惠
          vo.coupons.length > 0
              ? Container(
                  child: Row(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      Text('优惠',
                          style: TextStyle(
                              fontSize: 12.w, color: Color(0xFF999999))),
                      SizedBox(
                        width: 12.w,
                      ),
                      Container(
                          width: 280.w,
                          child: Wrap(
                            direction:Axis.horizontal,
                            alignment: WrapAlignment.start,
                            runSpacing:6.w,
                            children: List.generate(
                              vo.coupons.length,
                              (index) => getCouponWidget(
                                coupon: vo.coupons[index],
                              ),
                            ),
                          ))
                    ],
                  ),
                  margin: EdgeInsets.only(top: 12.w),
                )
              : Container(),

          //返利
          (vo.rebate ?? '').isNotEmpty
              ? Container(
                  child: Row(
                    children: [
                      Text('返利',
                          style: TextStyle(
                              fontSize: 12.w, color: Color(0xFF999999))),
                      SizedBox(
                        width: 12.w,
                      ),
                      Container(
                          padding: EdgeInsets.symmetric(
                              horizontal: 4.w, vertical: 1.w),
                          decoration: BoxDecoration(
                              border: Border.all(
                                  color: Color(0x80FF8000), width: 0.5.w),
                              borderRadius: BorderRadius.only(
                                  bottomRight: Radius.circular(8.0),
                                  topRight: Radius.circular(8.0),
                                  topLeft: Radius.circular(8.0))),
                          child: Text(vo.rebate!,
                              style: TextStyle(
                                  fontWeight: FontWeight.w600,
                                  fontSize: 11.w,
                                  color: Color(0xFFFF8000))))
                    ],
                  ),
                  margin: EdgeInsets.only(top: 12.w),
                )
              : Container(),
        ],
      ));
}

///
/// 商品信息
///
Widget buildProductNewlastWidget(
    {required Map<String, dynamic>? newInfo,
    void Function()? goBuy,
    void Function(Map<String, dynamic>)? onCouponTap}) {
  if (newInfo == null) {
    return Container();
  }
  List<dynamic>? couponInfo = [];
  if (newInfo['coupon_info'] is List) {
    couponInfo = newInfo['coupon_info'];
  }
  return Container(
    margin: EdgeInsets.only(top: 16.w),
    padding: EdgeInsets.only(left: 12.w, right: 12.w, top: 12.w, bottom: 10.w),
    decoration: BoxDecoration(
        boxShadow: [
          BoxShadow(
              color: Color(0x14000000),
              spreadRadius: 2.2,
              offset: Offset(0, 1),
              blurRadius: 3)
        ],
        color: Colors.white,
        borderRadius: BorderRadius.all(Radius.circular(8.0))),
    child: Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: [
        Text('商品最新信息',
            style: TextStyle(fontSize: 12.w, color: Color(0xFF999999))),
        SizedBox(
          height: 4.w,
        ),
        Row(
          crossAxisAlignment: CrossAxisAlignment.center,
          children: [
            Expanded(
              child: Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  Text('${newInfo['item_title']}',
                      maxLines: 1,
                      overflow: TextOverflow.ellipsis,
                      style: TextStyle(
                          color: Color(0xFF333333),
                          fontSize: 15.w,
                          fontWeight: FontWeight.w600)),
                  SizedBox(
                    height: 4.w,
                  ),
                  Text('${newInfo['end_price_text']}',
                      style: TextStyle(
                          color: Color(0xFFFF0400),
                          fontSize: 14.w,
                          fontWeight: FontWeight.w600)),
                ],
              ),
              flex: 1,
            ),
            InkWell(
                child: Container(
                  padding:
                      EdgeInsets.symmetric(vertical: 7.w, horizontal: 12.w),
                  child: Text('${newInfo['button_text']}',
                      style: TextStyle(
                          fontWeight: FontWeight.w600,
                          fontSize: 12.w,
                          color: Color(0xFF302600))),
                  decoration: BoxDecoration(
                      color: Color(0xFFFFCA00),
                      borderRadius: BorderRadius.circular(28.w)),
                ),
                onTap: goBuy),
          ],
        ),
        couponInfo!.isNotEmpty
            ? SizedBox(
                height: 4.w,
              )
            : Container(),
        couponInfo.isNotEmpty ? Divider() : Container(),
        couponInfo.isNotEmpty
            ? Row(
                children: [
                  Text('优惠',
                      style:
                          TextStyle(color: Color(0xFF999999), fontSize: 12.w)),
                  Expanded(
                    flex: 1,
                    child: Container(
                      child: ListView.builder(
                        shrinkWrap: true,
                        scrollDirection: Axis.horizontal,
                        itemCount: couponInfo.length,
                        padding: EdgeInsets.zero,
                        itemBuilder: (BuildContext context, int index) {
                          Map<String, dynamic> data = couponInfo![index];
                          return InkWell(
                            child: Container(
                                alignment: Alignment.centerLeft,
                                margin: EdgeInsets.only(left: 12.w),
                                height:16.w,
                                child: Row(
                                  crossAxisAlignment: CrossAxisAlignment.center,
                                  children: [
                                    Text(data['coupon_name'],
                                        style: TextStyle(
                                          height: 1.2,
                                            color: Color(0xFFFF0400),
                                            fontSize: 12.w)),
                                    Container(alignment:Alignment.centerLeft,width:14.w,height:10.w,child:Image.asset(
                                        'assets/images/ic_arrow_gray.png',
                                        package: 'price_module',
                                        width: 14.w,
                                        height: 10.w,
                                        color: Color(data['status'] == '1'
                                            ? 0xFFFF0400
                                            : 0xFF999999),),)
                                  ],
                                )),
                            onTap: () {
                              if (onCouponTap != null) {
                                onCouponTap(data);
                              }
                            },
                          );
                        },
                      ),
                      height: 18.w,
                    ),
                  ),
                ],
              )
            : Container(),
      ],
    ),
  );
}

///
/// 优惠卷
/// [onReceiveTap]                  点击领取优惠卷
///
Widget buildCouponWidget(BuildContext context,
    {required List<Coupon?> coupons,
    required String couponText,
    required Function(Coupon) onReceiveTap}) {
  if (coupons.isEmpty) {
    return Container();
  }

  ///
  /// 优惠卷全部过期
  ///
  if (couponText.isNotEmpty) {
    Coupon? coupon;
    try {
      coupon =
          coupons.singleWhere((ele) => '${ele?.status}' == '1', orElse: () {
        return null;
      });
    } catch (e) {
      print(e);
    }

    return InkWell(
        child: Container(
          margin: EdgeInsets.only(top: 12.w, left: 13.w, right: 13.w),
          decoration: BoxDecoration(
              color: Color(0xFFF5F5F5),
              borderRadius: BorderRadius.all(Radius.circular(8))),
          child: Column(
            children: [
              Container(
                margin: EdgeInsets.only(top: 10.w, left: 12.w),
                alignment: Alignment.centerLeft,
                child: Text(
                  '$couponText',
                  style: TextStyle(color: Color(0xFFFF8000), fontSize: 12.w),
                ),
              ),
              Row(crossAxisAlignment: CrossAxisAlignment.start, children: [
                Container(
                  alignment: Alignment.centerLeft,
                  margin: EdgeInsets.only(left: 12.w, top: 12.w),
                  child: Text('优惠:',
                      style:
                          TextStyle(fontSize: 14.w, color: Color(0xFF302600))),
                ),
                Expanded(
                    child: Container(
                      margin: EdgeInsets.only(top: 5.w, bottom: 10.w),
                      child: Wrap(
                        runSpacing:6.rpx,
                        children:
                            List<Widget>.generate(coupons.length, (index) {
                          Coupon coupon = coupons[index]!;
                          return Container(
                              child: Text(
                                '${coupon.name}',
                                style: TextStyle(
                                  color: Color(coupon.status == '2'
                                      ? 0xFF999999
                                      : 0xFFFF0400),
                                  fontSize: 12.w,
                                  fontWeight: FontWeight.w500,
                                ),
                              ),
                              margin: EdgeInsets.only(left: 8.w, top: 6.w),
                              padding: EdgeInsets.only(
                                  top: 2.w, bottom: 2.w, left: 4.w, right: 4.w),
                              decoration: BoxDecoration(
                                  border: Border.all(
                                      color: Color(coupon.status == '2'
                                          ? 0xFFFFFFFF
                                          : 0xFFFF0400),
                                      width: 0.5.w),
                                  borderRadius: BorderRadius.circular(6.w),
                                  color: Color(0xFFFFFFFF)));
                        }),
                      ),
                    ),
                    flex: 1),
                coupon == null
                    ? Container(
                        alignment: Alignment.centerRight,
                        margin: EdgeInsets.only(right: 11.w, top: 12.w),
                        child: Row(
                          mainAxisAlignment: MainAxisAlignment.center,
                          crossAxisAlignment: CrossAxisAlignment.center,
                          children: [
                            Text(
                              '已失效',
                              style: TextStyle(
                                  fontSize: 14.w, color: Color(0xFFFF0400)),
                            ),
                          ],
                        ),
                      )
                    : Container(
                        alignment: Alignment.centerRight,
                        margin: EdgeInsets.only(right: 11.w, top: 12.w),
                        child: Row(
                          mainAxisAlignment: MainAxisAlignment.center,
                          crossAxisAlignment: CrossAxisAlignment.center,
                          children: [
                            Text(
                              '领取',
                              style: TextStyle(
                                  fontSize: 14.w, color: Color(0xFF999999)),
                            ),
                            Container(
                              child: Image.asset(
                                'assets/images/ic_arrow_gray.png',
                                package: 'price_module',
                                width: 14.w,
                                height: 10.w,
                              ),
                              margin: EdgeInsets.only(top: 5.w),
                            )
                          ],
                        ),
                      ),
              ])
            ],
          ),
        ),
        onTap: coupon == null
            ? null
            : () {
                showModalBottomSheet(
                    context: context,
                    builder: (BuildContext context) {
                      return Container(
                          decoration: BoxDecoration(
                              color: Color(0xFFF5F5F5),
                              borderRadius: BorderRadius.vertical(
                                  top: Radius.circular(20))),
                          child: SafeArea(
                              child: Container(
                                  height: 284.w,
                                  decoration: BoxDecoration(
                                      color: Color(0xFFF5F5F5),
                                      borderRadius: BorderRadius.vertical(
                                          top: Radius.circular(20))),
                                  child: ListView.builder(
                                      itemCount: coupons.length + 1,
                                      itemBuilder:
                                          (BuildContext context, int index) {
                                        if (index == 0) {
                                          return Container(
                                              height: 50.w,
                                              margin:
                                                  EdgeInsets.only(bottom: 2.w),
                                              child: Stack(children: [
                                                Center(
                                                  child: Text(
                                                    '领取优惠券',
                                                    style: TextStyle(
                                                        color:
                                                            Color(0xFF333333),
                                                        fontWeight:
                                                            FontWeight.w500,
                                                        fontSize: 15.w),
                                                  ),
                                                ),
                                                Align(
                                                  child: IconButton(
                                                      icon: Image.asset(
                                                        'assets/images/ic_pop_close.png',
                                                        package: 'price_module',
                                                        width: 24.w,
                                                        height: 24.w,
                                                      ),
                                                      onPressed: () {
                                                        Navigator.maybePop(
                                                            context);
                                                      }),
                                                  alignment: Alignment.topRight,
                                                ),
                                              ]));
                                        }
                                        Coupon coupon = coupons[index - 1]!;
                                        return Container(
                                          margin: EdgeInsets.only(
                                              left: 13.w,
                                              right: 13.w,
                                              top: index > 1 ? 8.w : 0),
                                          padding: EdgeInsets.only(
                                              left: 12.w, right: 12.w),
                                          height: 48,
                                          alignment: Alignment.centerLeft,
                                          decoration: BoxDecoration(
                                              color: Color(0xFFFFFFFF),
                                              borderRadius: BorderRadius.all(
                                                  Radius.circular(8.w))),
                                          child: Row(children: [
                                            Expanded(
                                                child: Text(
                                              coupon.name!,
                                              style: TextStyle(
                                                  fontSize: 15.w,
                                                  fontWeight: FontWeight.w500,
                                                  color: Color(
                                                      coupon.status == '1'
                                                          ? 0xFFFF0400
                                                          : 0xFF999999)),
                                            )),
                                            InkWell(
                                              child: Container(
                                                alignment: Alignment.center,
                                                width: 60.w,
                                                height: 30.w,
                                                child: Text(
                                                  '领取',
                                                  style: TextStyle(
                                                      color: Color(
                                                          coupon.status == '1'
                                                              ? 0xFFFFFFFF
                                                              : 0xFF999999)),
                                                ),
                                                decoration: BoxDecoration(
                                                    color: Color(
                                                        coupon.status == '1'
                                                            ? 0xFFFF0400
                                                            : 0xFFE9E9E9),
                                                    borderRadius:
                                                        BorderRadius.all(Radius
                                                            .circular(16.w))),
                                              ),
                                              onTap: () {
                                                if (coupon.status == '1') {
                                                  onReceiveTap(coupon);
                                                  Navigator.maybePop(context);
                                                }
                                              },
                                            ),
                                          ]),
                                        );
                                      }))));
                    }).then((value) {});
              });
  }
  return InkWell(
      child: Container(
        margin: EdgeInsets.only(top: 12.w, left: 13.w, right: 13.w),
        decoration: BoxDecoration(
            color: Color(0xFFF5F5F5),
            borderRadius: BorderRadius.all(Radius.circular(8))),
        child: Row(crossAxisAlignment: CrossAxisAlignment.start, children: [
          Container(
            alignment: Alignment.centerLeft,
            margin: EdgeInsets.only(left: 12.w, top: 12.w),
            child: Text('优惠:',
                style: TextStyle(fontSize: 14.w, color: Color(0xFF302600))),
          ),
          Expanded(
              child: Container(
                margin: EdgeInsets.only(top: 5.w, bottom: 10.w),
                child: Wrap(
                  runSpacing: 6.rpx,
                  children: List<Widget>.generate(coupons.length, (index) {
                    Coupon coupon = coupons[index]!;
                    return Container(
                        child: Text(
                          '${coupon.name}',
                          style: TextStyle(
                            color: Color(
                                coupon.status == '2' ? 0xFF999999 : 0xFFFF0400),
                            fontSize: 12.w,
                            fontWeight: FontWeight.w500,
                          ),
                        ),
                        margin: EdgeInsets.only(left: 8.w, top: 6.w),
                        padding: EdgeInsets.only(
                            top: 2.w, bottom: 2.w, left: 4.w, right: 4.w),
                        decoration: BoxDecoration(
                            border: Border.all(
                                color: Color(coupon.status == '2'
                                    ? 0xFFFFFFFF
                                    : 0xFFFF0400),
                                width: 0.5.w),
                            borderRadius: BorderRadius.circular(6.w),
                            color: Color(0xFFFFFFFF)));
                  }),
                ),
              ),
              flex: 1),
          Container(
            alignment: Alignment.centerRight,
            margin: EdgeInsets.only(right: 11.w, top: 12.w),
            child: Row(
              mainAxisAlignment: MainAxisAlignment.center,
              crossAxisAlignment: CrossAxisAlignment.center,
              children: [
                Text(
                  '领取',
                  style: TextStyle(fontSize: 14.w, color: Color(0xFF999999)),
                ),
                Container(
                    child: Image.asset(
                  'assets/images/ic_arrow_gray.png',
                  package: 'price_module',
                  width: 14.w,
                  height: 10.w,
                ))
              ],
            ),
          ),
        ]),
      ),
      onTap: () {
        showModalBottomSheet(
            context: context,
            builder: (BuildContext context) {
              return Container(
                  decoration: BoxDecoration(
                      color: Color(0xFFF5F5F5),
                      borderRadius:
                          BorderRadius.vertical(top: Radius.circular(20))),
                  child: SafeArea(
                      child: Container(
                          height: 284.w,
                          decoration: BoxDecoration(
                              color: Color(0xFFF5F5F5),
                              borderRadius: BorderRadius.vertical(
                                  top: Radius.circular(20))),
                          child: ListView.builder(
                              itemCount: coupons.length + 1,
                              itemBuilder: (BuildContext context, int index) {
                                if (index == 0) {
                                  return Container(
                                      height: 50.w,
                                      margin: EdgeInsets.only(bottom: 2.w),
                                      child: Stack(children: [
                                        Center(
                                          child: Text(
                                            '领取优惠券',
                                            style: TextStyle(
                                                color: Color(0xFF333333),
                                                fontWeight: FontWeight.w500,
                                                fontSize: 15.w),
                                          ),
                                        ),
                                        Align(
                                          child: IconButton(
                                              icon: Image.asset(
                                                'assets/images/ic_pop_close.png',
                                                package: 'price_module',
                                                width: 24.w,
                                                height: 24.w,
                                              ),
                                              onPressed: () {
                                                Navigator.maybePop(context);
                                              }),
                                          alignment: Alignment.topRight,
                                        ),
                                      ]));
                                }
                                Coupon coupon = coupons[index - 1]!;
                                return Container(
                                  margin: EdgeInsets.only(
                                      left: 13.w,
                                      right: 13.w,
                                      top: index > 1 ? 8.w : 0),
                                  padding:
                                      EdgeInsets.only(left: 12.w, right: 12.w),
                                  height: 48,
                                  alignment: Alignment.centerLeft,
                                  decoration: BoxDecoration(
                                      color: Color(0xFFFFFFFF),
                                      borderRadius: BorderRadius.all(
                                          Radius.circular(8.w))),
                                  child: Row(children: [
                                    Expanded(
                                        child: Text(
                                      coupon.name!,
                                      style: TextStyle(
                                          fontSize: 15.w,
                                          fontWeight: FontWeight.w500,
                                          color: Color(coupon.status == '1'
                                              ? 0xFFFF0400
                                              : 0xFF999999)),
                                    )),
                                    InkWell(
                                      child: Container(
                                        alignment: Alignment.center,
                                        width: 60.w,
                                        height: 30.w,
                                        child: Text(
                                          '领取',
                                          style: TextStyle(
                                              color: Color(coupon.status == '1'
                                                  ? 0xFFFFFFFF
                                                  : 0xFF999999)),
                                        ),
                                        decoration: BoxDecoration(
                                            color: Color(coupon.status == '1'
                                                ? 0xFFFF0400
                                                : 0xFFE9E9E9),
                                            borderRadius: BorderRadius.all(
                                                Radius.circular(16.w))),
                                      ),
                                      onTap: () {
                                        if (coupon.status == '1') {
                                          onReceiveTap(coupon);
                                          Navigator.maybePop(context);
                                        }
                                      },
                                    ),
                                  ]),
                                );
                              }))));
            }).then((value) {});
      });
}

///
/// 商品图文详情
/// [html]                          商品图文信息
///
Widget buildProductDetailsWidget(BuildContext context,
    {required String html}) {
  return Container(
    margin: EdgeInsets.only(top: 16.w, bottom: 4.w, left: 4.w, right: 4.w),
    child: Html(
        data: html,
        style: {},
        customRender: {},
        onLinkTap:(String? url,RenderContext context,Map<String, String> attributes,dom.Element? element) {
          print("=========>onLinkTap $url");
        },
        onImageTap:(String? url,RenderContext context,Map<String, String> attributes,dom.Element? element) {
          print("=========>onImageTap $url");
        },
        onImageError: (exception, stackTrace) {
          print("=========>onImageError $stackTrace");
        }),
  );
}

///
/// 提示信息
/// [onReportTap]               举报按钮
///
Widget buildPromptWidget(
    {required String? releaseTime,
    required String? checkText,
    VoidCallback? onReportTap,
    Map<String, dynamic>? extras}) {
  return Container(
      child: Column(
    children: [
      Container(
          margin: EdgeInsets.only(bottom: 8),
          child: Row(children: [
            Expanded(
                child: Container(
                    child: Text(
                  '${releaseTime ?? '5s前'}发布 ｜ ${checkText ?? '检查有效'}',
                  style: TextStyle(
                      fontWeight: FontWeight.w400,
                      fontSize: 12.w,
                      color: Color(0xFF666666)),
                )),
                flex: 1),
            _buildIconText(
              image: 'assets/images/ic_report.png',
              width: 16.w,
              text: '优惠举报',
              onTap: onReportTap,
              style: TextStyle(fontSize: 12.w, color: Color(0xFFFF8000)),
              margin: EdgeInsets.only(left: 10.w),
            )
          ])),
      Container(
        padding: EdgeInsets.only(top: 7.w, bottom: 12.w),
        decoration: BoxDecoration(
            border: Border(
                top: BorderSide(color: Color(0xFFE9E9E9), width: 0.5.w))),
        child: Text(
          '折扣信息来自于平台推荐和网络，购买前请核实促销，若发现优惠过期或不实信息，请使用优惠举报功能',
          style: TextStyle(fontSize: 12.w, color: Color(0xFF999999)),
        ),
      )
    ],
  ));
}

///
/// 相关优惠信息
///
Widget buildRelatedDiscountWidget(
    {List<ProductItemEntity>? productEntitys,
    void Function(ProductItemEntity, int)? onItemTap,
    void Function()? onMoreTap}) {
  bool isShowMore = false;
  if (productEntitys == null || productEntitys.length <= 0) {
    return Container();
  } else if (productEntitys.length > 6) {
    isShowMore = true;
    productEntitys = productEntitys.getRange(0, 6).toList();
  }
  return Container(
    decoration: BoxDecoration(
        color: Color(0xFFF5F5F5),
        borderRadius: BorderRadius.vertical(top: Radius.circular(8.w))),
    child: Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: [
        Container(
          margin:
              EdgeInsets.only(left: 13.w, right: 13.w, top: 12.w, bottom: 4.w),
          child: Text(
            '相关优惠',
            style: TextStyle(),
          ),
        ),
        PriceListWidget(
          shrinkWrap: true,
          padding: EdgeInsets.only(left: 13.w, right: 13.w),
          list: productEntitys,
          physics: NeverScrollableScrollPhysics(),
          onItemTap: onItemTap,
        ),

        ///查看更多
        isShowMore
            ? InkWell(
                child: Container(
                  padding: EdgeInsets.symmetric(vertical: 11.w),
                  child: Row(
                    mainAxisAlignment: MainAxisAlignment.center,
                    children: [
                      Text('查看更多优惠',
                          style: TextStyle(
                              color: Color(0xFF999999), fontSize: 13.w)),
                      Icon(
                        Icons.keyboard_arrow_right,
                        size: 16.w,
                        color: Color(0xFF999999),
                      )
                    ],
                  ),
                ),
                onTap: onMoreTap,
              )
            : Container(height: 8.w, color: Color(0xFFF5F5F5))
      ],
    ),
  );
}

///
/// 分享弹窗
///
///
Future<T?> showShareModalBottom<T>(BuildContext context,{required List<Map<String, dynamic>> items,Map<String, dynamic> extras = const {}}) {
  return showPopBottomSheet(context:context,pageName:RoutePath.PRICE_DETAIL_PAGE.path(),items: items,extras: extras);
}

///
/// 举报弹窗
/// [context]                   上下文对象
/// [items]                     菜单列表
///
Future<T?> showReceiveModalBottom<T>(BuildContext context,
    {required List<ReportModel> items}) {
  return _showModalBottomList(context, itemCount: items.length,
      itemBuilder: (BuildContext context, int index) {
    Widget widget;
    ReportModel? reportModel;
    if (index == items.length) {
      widget = Container(
        color: Colors.white,
        alignment: Alignment.center,
        height: 49,
        margin: EdgeInsets.only(top: 8),
        child: Text(
          '取消',
          style: TextStyle(color: Color(0xFF666666), fontSize: 15),
        ),
      );
    } else {
      reportModel = items[index];
      widget = Container(
        decoration: BoxDecoration(
            color: Colors.white,
            borderRadius: index == 0
                ? BorderRadius.vertical(top: Radius.circular(20))
                : null),
        alignment: Alignment.center,
        height: 49,
        child: Text(
          items[index].text!,
          style: TextStyle(fontSize: 15, color: Color(0xFF302600)),
        ),
        margin: EdgeInsets.only(top: index == 0 ? 0 : 0.6),
      );
    }
    return InkWell(
      child:widget,
      onTap: () {
        Navigator.pop(context, reportModel);
      },
    );
  });
}

///
/// 底部导航栏目
/// [like]                      喜欢的个数
/// [favorites]                 收藏的个数
/// [onLikeTap]                 喜欢
/// [onFavoritesTap]            收藏
/// [onBuyTap]                  购买
///
Widget buildBottomNavWidget(
    {required String like,
    required String favorites,
    bool isLike = false,
    bool isFavorites = false,
    required String text,
    VoidCallback? onLikeTap,
    VoidCallback? onFavoritesTap,
    VoidCallback? onBuyTap,
    VoidCallback? onShareTap,
    String? status,
    Map<String, dynamic>? extras,
    required List<String> favoritesImages,
    required ValueNotifier<int> favoritesValueNotifier,
    required List<String> likeImages,
    required ValueNotifier<int> likeValueNotifier}) {
  if (isLike) {
    likeValueNotifier.value = likeImages.length - 1;
  } else {
    likeValueNotifier.value = 0;
  }
  if (isFavorites) {
    favoritesValueNotifier.value = favoritesImages.length - 1;
  } else {
    favoritesValueNotifier.value = 0;
  }
  return SafeArea(
    child: Container(
        height: 49.w,
        padding: EdgeInsets.only(left: 13, right: 13.w),
        child: Row(
          children: [
            Row(
              children: [
                _buildAnimationText(
                  images: likeImages,
                  valueNotifier: likeValueNotifier,
                  text: like,
                  onTap: onLikeTap,
                  margin: EdgeInsets.only(right: 10.w),
                  width: 24.w,
                  height: 24.w,
                  style:TextStyle(fontSize: 11.w, color: Color(0xFF999999)),
                ),
                _buildAnimationText(
                  images: favoritesImages,
                  valueNotifier: favoritesValueNotifier,
                  text: favorites,
                  margin: EdgeInsets.only(right: 10.w, left: 10.w),
                  onTap: onFavoritesTap,
                  width: 24.w,
                  height: 24.w,
                  style:TextStyle(fontSize: 11.w, color: Color(0xFF999999)),
                ),
              ],
            ),
            AuditModeUtils.getInstance()!.isAuditMode()?Container(width:1):InkWell(
                child: Container(
                  margin: EdgeInsets.only(right: 24.w, left: 10.w),
                  child: Column(
                    mainAxisAlignment: MainAxisAlignment.center,
                    children: [
                      Image.asset('assets/images/ic_btn_share1.png',package: 'price_module',width: 24.w,height: 24.w,),
                      Text('分享', style:TextStyle(fontSize: 11.w, color: Color(0xFF999999)))
                    ],
                  ),
                ),
                onTap: onShareTap,
              ),
            Expanded(
              child: Container(
                height: 44.w,
                child: InkWell(
                  child: Container(
                    decoration: BoxDecoration(color: Color((status == null || status == '2') ? 0xFFFFCA00 : 0xFFE9E9E9),borderRadius:BorderRadius.all(Radius.circular(22.w))),
                    margin: EdgeInsets.symmetric(vertical: 2),
                    alignment: Alignment.center,
                    child: Text(text,style: TextStyle(color: Color(0xFF302600),fontSize: 15.w,fontWeight: FontWeight.w600))
                  ),
                  onTap: status == '2' ? onBuyTap : null,
                )
              ),
              flex: 1,
            ),
          ],
        )),
  );
}

Widget _buildAnimationText({
  required String text,
  ValueNotifier<int>? valueNotifier,
  double? width1,
  EdgeInsetsGeometry? margin,
  String package = 'price_module',
  List<String>? images,
  double? width,
  double? height,
  VoidCallback? onTap,
  TextStyle? style,
}) {
  return Container(
      padding: margin,
      width: width1,
      alignment: Alignment.centerLeft,
      child: InkWell(
        child: Column(mainAxisAlignment: MainAxisAlignment.center, children: [
          ImageAnimationWidget(
            valueNotifier: valueNotifier,
            images: images,
            package: package,
            width: width,
            height: height,
          ),
          Text(text, style: style)
        ]),
        onTap: onTap,
      ));
}

///
/// 图标文字按钮
/// [image]     图片
/// [text]      文字
/// [onTap]     点击事件
///
Widget _buildIconText(
    {required String image,
    required String text,
    double width = 24,
    double? width1,
    EdgeInsetsGeometry? margin,
    TextStyle? style,
    VoidCallback? onTap}) {
  return Container(
      width: width1,
      padding: margin,
      alignment: Alignment.center,
      child: InkWell(
        child: Row(children: [
          ImageWidget(
            name: image,
            width: width,
          ),
          Container(
            child: Text(text, style: style),
            margin: EdgeInsets.only(left: 4.w),
          )
        ]),
        onTap: onTap,
      ));
}

///
/// 自定义底部的导航条
///
SwiperPlugin _buildCustomPagination() {
  return SwiperCustomPagination(
      builder: (BuildContext context, SwiperPluginConfig? config) {
    return Positioned(
        bottom: 12.w,
        right: 12.w,
        child: Container(
          width: 43.w,
          height: 21.w,
          alignment: Alignment.center,
          decoration: BoxDecoration(
              color: Color.fromRGBO(48, 38, 0, 0.4),
              borderRadius: BorderRadius.circular(10.w)),
          child: Text(
            '${config!.activeIndex! + 1}/${config.itemCount}',
            style: TextStyle(
                color: Colors.white,
                fontSize: 12.w,
                fontWeight: FontWeight.w500),
          ),
        ));
  });
}

Future<T?> _showModalBottomList<T>(BuildContext context,
    {required int itemCount, required IndexedWidgetBuilder itemBuilder}) {
  return showModalBottomSheet(
      context: context,
      builder: (BuildContext context) {
        return Container(
            decoration: BoxDecoration(
                color: Colors.white,
                borderRadius: BorderRadius.vertical(top: Radius.circular(20))),
            child: SafeArea(
                child: Container(
              decoration: BoxDecoration(
                  color: Color(0xFFF5F5F5),
                  borderRadius:
                      BorderRadius.vertical(top: Radius.circular(20))),
              child: ListView.builder(
                  shrinkWrap: true,
                  primary: true,
                  itemCount: itemCount + 1,
                  physics: NeverScrollableScrollPhysics(),
                  itemBuilder: itemBuilder),
            )));
      });
}
