import 'package:flutter/material.dart';
import 'package:price_module/page/model/index.dart';
import 'package:price_module/utils/event_bus_utils.dart';
import 'package:common_module/utils/xapp_utils.dart';

typedef OverlayTapUpCallback = void Function(OverlayWidgetBuilder overlayBuilder);

typedef OverlayWidgetBuilder = void Function(Widget? overlayWidget);


class OtherWidget extends StatefulWidget {
  final Widget child;
  final OverlayTapUpCallback? tapUpCallback;
  final GestureTapCallback? onCloseTap;
  final Widget Function()? onHeaderWidget;

  const OtherWidget({Key? key,
    required this.child,
    this.tapUpCallback,
    this.onCloseTap,
    required this.onHeaderWidget,
  }) : super(key: key);

  @override
  _OtherWidgetState createState() => _OtherWidgetState();
}

class _OtherWidgetState extends State<OtherWidget> {
  GlobalKey globalKey = GlobalKey();

  OverlayEntry? overlayEntry;

  void onCloseEntry({bool? close}) {
    if (overlayEntry != null) {
      overlayEntry!.remove();
      overlayEntry = null;
    }
    if(widget.onCloseTap != null){
      widget.onCloseTap!();
    }
  }

  void showOverlayEntry({required double height,required double headerHeight ,required Widget overlayWidget}) {
    if(overlayEntry != null){
      overlayEntry!.remove();
      overlayEntry = null;
    }
    overlayEntry = new OverlayEntry(maintainState: true,builder: (BuildContext context) {
      return Container(child:Material(child:Column(children: [
        InkWell(child:Container(height:height ,color:Colors.transparent),onTap:onCloseEntry),
        Container(
          alignment: Alignment.center,
          child:widget.onHeaderWidget!(),height:headerHeight,
          color: Colors.transparent,
        ),
        Container(child:overlayWidget,color:Color(0xFF000000).withAlpha(102)),
        Expanded(
          child: InkWell(child: Container(color:Color(0xFF000000).withAlpha(102)),onTap: onCloseEntry),
          flex: 1,
        ),
      ],),color:Colors.transparent,));
    });
    Overlay.of(context,rootOverlay: true)!.insert(overlayEntry!);
  }

  @override
  void initState() {
    eventBus.on<String>().listen((event) {
      if(event == 'close'){
        onCloseEntry(close:true);
      }
    });
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return InkWell(
      key: globalKey,
      child: Container(child:widget.child,alignment:Alignment.center,),
      onTap:(){
        if (overlayEntry != null) {
          overlayEntry!.remove();
          overlayEntry = null;
        }
        RenderBox renderBox = context.findRenderObject() as RenderBox;
        Offset offset = renderBox.localToGlobal(Offset.zero);
        widget.tapUpCallback!((Widget? overlayWidget){
          if(overlayWidget == null){
            onCloseEntry(close:true);
          }else{
            showOverlayEntry(height: offset.dy,headerHeight:renderBox.size.height,overlayWidget:overlayWidget);
          }
        });
      },
    );
  }


  @override
  void dispose() {
    super.dispose();
  }
}

class OverlayEntryHeader extends StatefulWidget {
  final List<TabVo> tabList;
  final String? Function(TabVo)? onHeaderTitle;
  final void Function(int,TabVo)? onTap;
  const OverlayEntryHeader({ Key? key,required this.tabList,this.onHeaderTitle,this.onTap}) : super(key: key);
  @override
  _OverlayEntryHeaderState createState() => _OverlayEntryHeaderState();
}

class _OverlayEntryHeaderState extends State<OverlayEntryHeader> {

  late List<TabVo> tabList;

  @override
  void initState() {
    tabList = widget.tabList;
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Row(
      children:List.generate(tabList.length,(index){
         TabVo tabVo = tabList[index];
         String? title = widget.onHeaderTitle!(tabVo);
         return Expanded(
           child:InkWell(child:Container(
             alignment: Alignment.center,
             child:Row(mainAxisAlignment:MainAxisAlignment.center,children: [
               Container(child:Text(
                 '$title',
                 maxLines: 1,
                 overflow: TextOverflow.ellipsis,
                 style: TextStyle(color: Color((tabVo.selected != null || tabVo.open)? 0xFFFF8000: 0xFF302600),fontSize: 13.w)
                ),constraints: BoxConstraints(maxWidth: 65.w),margin: EdgeInsets.only(right: 4.w),),
                                    Image.asset(
                                      'assets/images/${tabVo.open ? 'ic_arrow_up' : 'ic_arrow_down'}.png',
                                      package: 'price_module',
                                      width: 6.w,
                                      height: 4.w,
                                    )
                                  ],)
                                ),onTap:(){
                                  widget.onTap!(index,tabVo);
                                },),
                                flex:1,
                              );
      })
    );
  }
}