import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:price_module/framework/base/ds_provider.dart';
import 'package:price_module/framework/base/ds_provider_widget.dart';
import 'package:price_module/framework/model/ds_model.dart';
import 'package:price_module/page/action/index.dart';
import 'package:price_module/page/model/index.dart';
import 'package:price_module/page/widget/index.dart';
import 'package:common_module/utils/xapp_utils.dart';
import 'package:price_module/widget/price/price_item.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'package:xiaoxiong_repository/entity/product_item_entity.dart';

///
/// 代理类，类似接口
///
abstract class SearchWidgetDelegate {
  ///
  /// 好价关键词搜索
  /// keywords       待搜索的关键词
  ///
  void onKeywordsSearch(String keywords);

  ///
  /// 设置页面isGrid
  ///
  void setPageGrid({bool? isGrid});
}

///
/// 好价搜索页面
///
class SearchWidget extends StatefulWidget {
  final bool isGrid;
  final List<dynamic> priceCategory;
  final Map<int, dynamic> platformIconMap;
  final String keywords;
  final void Function(Map<String, dynamic>)? onGoodDetails;
  final void Function(SearchWidgetDelegate)? onWidgetDelegate;
  const SearchWidget(
      {Key? key,
      required this.priceCategory,
      required this.keywords,
      required this.platformIconMap,
      this.isGrid = true,
      this.onGoodDetails,
      this.onWidgetDelegate})
      : super(key: key);

  @override
  _SearchWidgetState createState() => _SearchWidgetState();
}

class _SearchWidgetState extends State<SearchWidget>
    with
        AutomaticKeepAliveClientMixin,
        SearchWidgetDelegate,
        IPriceActionDelegate {
  RefreshController _refreshController = RefreshController();

  IPriceAction<PriceVo>? _priceAction;

  @override
  bool get wantKeepAlive => true;

  @override
  void initState() {
    if (widget.onWidgetDelegate != null) {
      widget.onWidgetDelegate!(this);
    }
    _priceAction = PriceAction(delegate: this);
    _searchKeywords = widget.keywords;
    _priceAction!.setPriceCategory(
        priceCategory: widget.priceCategory,
        keywords: _searchKeywords,
        platformIconMap: widget.platformIconMap);
    super.initState();
  }

  @override
  void setPageGrid({bool? isGrid}) {
    _priceAction!.getProductListVo().isGrid = isGrid;
    _priceAction!.getProductListVo().notifyListener(true);
  }

  String? _searchKeywords;

  @override
  void onKeywordsSearch(String keywords) {
    if (_searchKeywords != keywords) {
      _searchKeywords = keywords;
      _priceAction!.qryProductListEvent(keywords: keywords);
    }
  }

  ScrollController controller = ScrollController();
  @override
  Widget build(BuildContext context) {
    super.build(context);
    return DSProviderWidget<PriceVo, IPriceAction<PriceVo>>(
        dsAction: _priceAction!,
        builder: (BuildContext context, child) {
          return Container(
              color: Color(0xFFF5F5F5),
              child: buildCustomScroll(
                  onRefresh: null,
                  onLoad: () {
                    _priceAction?.loadMore();
                  },
                  onNotification: (ScrollNotification data) {
                    return true;
                  },
                  controller: controller,
                  refreshController: _refreshController,
                  children: [
                    //Tab分类Widget
                    buildSortHeader(
                        action: _priceAction!, keywords: widget.keywords),
                    //商品列表
                    DSProvider.value(
                        vo: _priceAction!.getProductListVo(),
                        builderWidget: (BuildContext context,
                            ProductListVo productListVo, Widget? widget1) {
                          if (productListVo.productList.isEmpty) {
                            if (productListVo.viewState == DSViewState.busy) {
                              return SliverToBoxAdapter(
                                  child: Container(
                                margin: EdgeInsets.only(top: 120.w),
                                child: CupertinoActivityIndicator(
                                  animating: true,
                                  radius: 16,
                                ),
                              ));
                            }
                            return SliverToBoxAdapter(
                                child: Container(
                                    margin: EdgeInsets.only(top: 100.w),
                                    child: Column(
                                      children: [
                                        Image.asset(
                                          'assets/zanwushuju.png',
                                          width: 261.w,
                                          height: 165.w,
                                          fit: BoxFit.cover,
                                        ),
                                        SizedBox(height: 16.w),
                                        Text('暂无数据',
                                            style: new TextStyle(
                                                color: Color(0xFF999999),
                                                fontSize: 14.w)),
                                      ],
                                    )));
                          } else {
                            if (productListVo.isGrid ?? false) {
                              return SliverToBoxAdapter(
                                  child: PriceGridWidget(
                                      shrinkWrap: true,
                                      padding: EdgeInsets.all(13.w),
                                      physics: NeverScrollableScrollPhysics(),
                                      list: productListVo.productList,
                                      onItemTap: (int index) {
                                        ProductItemEntity entity =
                                            productListVo.productList[index];
                                        Map<String, dynamic> goodDetails = {
                                          'productId': entity.id,
                                          'src': entity.itemPic,
                                          'title': entity.itemTitle,
                                          'remark': entity.special
                                        };
                                        if (widget.onGoodDetails != null) {
                                          widget.onGoodDetails!(goodDetails);
                                        }
                                      }));
                            } else {
                              return SliverToBoxAdapter(
                                  child: PriceListWidget(
                                shrinkWrap: true,
                                padding: EdgeInsets.only(
                                    left: 13.w, right: 13.w, bottom: 13.w),
                                physics: NeverScrollableScrollPhysics(),
                                list: productListVo.productList,
                                onItemTap:
                                    (ProductItemEntity entity, int index) {
                                  Map<String, dynamic> goodDetails = {
                                    'productId': entity.id,
                                    'src': entity.itemPic,
                                    'title': entity.itemTitle,
                                    'remark': entity.special
                                  };
                                  if (widget.onGoodDetails != null) {
                                    widget.onGoodDetails!(goodDetails);
                                  }
                                },
                              ));
                            }
                          }
                        }),
                  ]));
        });
  }

  //Tab分类Widget
  Widget buildSortHeader(
      {required IPriceAction<PriceVo> action, String? keywords}) {
    return DSProvider.value(
        vo: action.getTabListVo(),
        builderWidget:
            (BuildContext context, TabListVo tabListVo, Widget? widget) {
          return buildTapWidget(context, tabList: tabListVo.tabList,
              onChange: (index, enable) {
            tabListVo.tabList[index].open = enable;
            tabListVo.notifyListener(true);
          }, onNodeChange: (int? index, TabVo tabVo) {
            tabVo.open = false;
            if (tabVo.selected != null && tabVo.selected!['is_default'] == '1') {
              tabVo.selected = null;
            }
            tabListVo.notifyListener(true);
            _priceAction!.qryProductListEvent(keywords: keywords);
          });
        });
  }

  @override
  void onLoadComplete() {
    _refreshController.loadComplete();
    _refreshController.refreshCompleted();
  }

  @override
  void onloadFailed({String? msg}) {
    _refreshController.loadFailed();
    _refreshController.refreshCompleted(resetFooterState: true);
  }
}
