import 'package:flutter_boost/flutter_boost.dart';

///
/// Boost导航路由管理器
///
class NavigateUtils {
  ///
  /// 开启新页面统一API
  /// [path]              待打开页面的路径
  /// [isNative]          是否需伴随原生容器弹出，默认false
  /// [needLogin]         是否需要登录
  /// [arguments]         携带到下一页面的参数
  ///
  ///
  static Future<Map<dynamic, dynamic>> push({
    required String path,
    bool isNative = false,
    bool needLogin = true,
    Map<String, dynamic> arguments = const {},
  }) async {
    // 记录上一个页面的名称
    arguments['previousRouteName'] = _getPreviousRouteName();
    dynamic result = await BoostNavigator.instance.push(
      path,
      withContainer: isNative,
      arguments: arguments,
    );
    return result;
  }

  ///
  /// 关闭当前页面
  /// [arguments]               关闭页面时携带的参数
  ///
  static Future<bool> pop({Map<String, dynamic> arguments = const {}}) async {
    dynamic result = BoostNavigator.instance.pop(arguments);
    return result;
  }

  ///
  /// 回到主页
  /// [rootPath]            根路由路径
  ///
  static popRoot({
    required String rootPath,
  }) async {
    await BoostNavigator.instance.pop().then((value) async {
      PageInfo? pageInfo = BoostNavigator.instance.getTopPageInfo();
      String pageName = pageInfo?.pageName ?? '';
      if (pageName.isNotEmpty && pageName != rootPath) {
        await popRoot(rootPath: rootPath);
      }
    }).whenComplete(() async {});
  }

  // 获取下一个新页面的上一个页面名称(页面路径)
  static String? _getPreviousRouteName() {
    PageInfo? pageInfo = BoostNavigator.instance.getTopPageInfo();
    var boostPath = pageInfo?.pageName;
    return boostPath;
  }
}
