import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:price_module/eventbus/notify_event_type.dart';
import 'package:price_module/utils/event_bus_utils.dart';
import 'package:common_module/utils/xapp_utils.dart';

class LoadingPage extends StatelessWidget {
  final String msg;
  final int length;
  final EdgeInsetsGeometry? padding;
  const LoadingPage({Key? key, this.padding, this.msg = '1', this.length = 6})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return ListView.builder(
      padding: padding,
      itemCount: length,
      shrinkWrap: true,
      physics: NeverScrollableScrollPhysics(),
      itemBuilder: (BuildContext context, int index) {
        return _buildItem(index);
      },
    );
  }

  Widget _buildItem(int index) {
    return Container(
      decoration: BoxDecoration(
          color: Colors.white,
          borderRadius: BorderRadius.all(Radius.circular(8.w))),
      height: 120.w,
      margin: index > 0 ? EdgeInsets.only(top: 8.w) : null,
      child: Row(
        children: [
          Container(
            decoration: BoxDecoration(
                color: Color(0xFFE9E9E9),
                borderRadius:
                    BorderRadius.horizontal(left: Radius.circular(8.w))),
            height: 120.w,
            width: 120.w,
          ),
          Container(
            decoration: BoxDecoration(
                borderRadius:
                    BorderRadius.horizontal(right: Radius.circular(8.w))),
            padding: EdgeInsets.only(top: 8.w, left: 12.w),
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                Container(
                  constraints: BoxConstraints(maxWidth: 205.w),
                  height: 14.w,
                  color: Color(0xFFF5F5F5),
                ),
                SizedBox(
                  height: 4.w,
                ),
                Container(
                  constraints: BoxConstraints(maxWidth: 124.w),
                  height: 14.w,
                  color: Color(0xFFF5F5F5),
                ),
                SizedBox(
                  height: 13.w,
                ),
                Container(
                  constraints: BoxConstraints(maxWidth: 40.w),
                  height: 14.w,
                  color: Color(0xFFF5F5F5),
                ),
                SizedBox(
                  height: 8.w,
                ),
                Container(
                  constraints: BoxConstraints(maxWidth: 72.w),
                  height: 14.w,
                  color: Color(0xFFF5F5F5),
                ),
                SizedBox(
                  height: 8.w,
                ),
                Container(
                  height: 14.w,
                  child: Row(
                    children: [
                      Container(
                        width: 43.w,
                        height: 14.w,
                        color: Color(0xFFF5F5F5),
                      ),
                      Container(
                        constraints: BoxConstraints(
                          maxWidth: 119.w,
                        ),
                      ),
                      Container(
                        width: 43.w,
                        height: 14.w,
                        color: Color(0xFFF5F5F5),
                      ),
                    ],
                  ),
                )
              ],
            ),
          ),
        ],
      ),
    );
  }
}

///
/// 小熊加载失败界面
/// [margin]                边距
/// [errorMsg]              错误信息
/// [onContactTap]          联系我们按钮
/// [onReloadTap]           重新加载按钮
///
class FailedLoadPage extends StatelessWidget {
  final EdgeInsetsGeometry? margin;
  final String errorMsg;
  final void Function()? onContactTap;
  final void Function()? onReloadTap;
  const FailedLoadPage(
      {Key? key,
      this.margin,
      this.errorMsg = '加载失败，请刷新重试',
      this.onContactTap,
      this.onReloadTap})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Container(
      margin: margin,
      alignment: Alignment.topCenter,
      child: Column(
        children: [
          Container(
            width: 261.w,
            height: 165.w,
            child: Image.asset('assets/images/img_ empty_data.png',
                package: 'price_module'),
          ),
          SizedBox(
            height: 16.w,
          ),
          Text(
            errorMsg,
            style: TextStyle(fontSize: 14.w, color: Color(0xFF999999)),
          ),
          SizedBox(
            height: 24.w,
          ),
          Row(
            mainAxisAlignment: MainAxisAlignment.center,
            children: [
              _buildButtonWidget(
                  text: '联系客服',
                  style: TextStyle(
                      fontSize: 16.w,
                      color: Color(0xFF666666),
                      fontWeight: FontWeight.w600),
                  decoration: BoxDecoration(
                      borderRadius: BorderRadius.all(Radius.circular(8.w)),
                      border:
                          Border.all(width: 0.5.w, color: Color(0xFF999999))),
                  onTap: onContactTap ??
                      () {
                        ///首页中有监听,发送到监听客服
                        eventBus.fire(NotifyEventType(
                            eventType: NotifyEventEnum.MeiQiaEnum));
                      }),
              SizedBox(
                width: 12.w,
              ),
              _buildButtonWidget(
                  text: '重新加载',
                  style: TextStyle(
                      fontSize: 16.w,
                      color: Color(0xFF302600),
                      fontWeight: FontWeight.w600),
                  decoration: BoxDecoration(
                      borderRadius: BorderRadius.all(Radius.circular(8.w)),
                      color: Color(0xFFFFCA00)),
                  onTap: onReloadTap),
            ],
          )
        ],
      ),
    );
  }

  Widget _buildButtonWidget(
      {required String text,
      required TextStyle style,
      Decoration? decoration,
      void Function()? onTap}) {
    return InkWell(
      child: Container(
        width: 164.w,
        height: 40.w,
        child: Text(text, style: style),
        alignment: Alignment.center,
        decoration: decoration,
      ),
      onTap: onTap,
    );
  }
}
