import 'package:cached_network_image/cached_network_image.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_staggered_grid_view/flutter_staggered_grid_view.dart';
import 'package:xiaoxiong_repository/entity/product_item_entity.dart';
import 'package:common_module/utils/xapp_utils.dart';

class PriceGridWidget extends StatelessWidget {
  final EdgeInsetsGeometry padding;
  final bool shrinkWrap;
  final ScrollPhysics? physics;
  final void Function(int)? onItemTap;
  final List<ProductItemEntity>? list;
  final bool isHaoJia;
  const PriceGridWidget(
      {Key? key,
      this.padding = const EdgeInsets.all(0),
      this.shrinkWrap = false,
      this.physics,
      this.onItemTap,
      this.list,
      this.isHaoJia = false})
      : super(key: key);
  @override
  Widget build(BuildContext context) {
    return StaggeredGridView.countBuilder(
      shrinkWrap: shrinkWrap,
      physics: physics,
      padding: padding,
      crossAxisCount: 4,
      staggeredTileBuilder: (index) => StaggeredTile.fit(2),
      mainAxisSpacing: 8.w,
      crossAxisSpacing: 9.w,
      itemCount: list!.length,
      itemBuilder: itemGridBuilder,
    );
  }

  Widget itemGridBuilder(BuildContext context, int index) {
    ProductItemEntity vo = list![index];
    return InkWell(
        onTap: () => onItemTap!(index),
        child: Container(
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              Container(
                width: 170.w,
                height: 170.w,
                decoration: BoxDecoration(
                    color: Colors.white,
                    borderRadius:
                        BorderRadius.vertical(top: Radius.circular(8.w))),
                child: Stack(
                  alignment: Alignment.bottomLeft,
                  children: [
                    ClipRRect(
                      child: CachedNetworkImage(
                        imageUrl: vo.itemPic!,
                        placeholder: (BuildContext c, String s) {
                          return Image.asset(
                            'assets/images/placeholder_image.png',
                            package: 'price_module',
                          );
                        },
                      ),
                      borderRadius:
                          BorderRadius.vertical(top: Radius.circular(8.w)),
                    ),
                    Container(
                      height: 16.w,
                      margin: EdgeInsets.only(left: 8.w, bottom: 6.w),
                      padding: EdgeInsets.symmetric(horizontal: 4.w),
                      decoration: BoxDecoration(
                          color: Color(0xCCFFFFFF),
                          border: Border.all(
                              width: 0.5.w, color: Color(0xFFE9E9E9)),
                          borderRadius: BorderRadius.all(Radius.circular(3.w))),
                      child: Text(
                        vo.platform!,
                        style: TextStyle(
                          color: Color(0xFF666666),
                          fontSize: 10.w,
                        ),
                        textAlign: TextAlign.center,
                      ),
                    )
                  ],
                ),
              ),
              Container(
                decoration: BoxDecoration(
                    color: Colors.white,
                    borderRadius:
                        BorderRadius.vertical(bottom: Radius.circular(8.w))),
                padding: EdgeInsets.all(8.w),
                child: Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: [
                    Text(
                      vo.itemTitle!,
                      maxLines: 2,
                      style: TextStyle(
                          fontWeight: FontWeight.w600, fontSize: 14.w),
                      overflow: TextOverflow.ellipsis,
                    ),
                    vo.descriptionText!.isEmpty
                        ? Container()
                        : SizedBox(height: 4.w),
                    vo.descriptionText!.isEmpty
                        ? Container()
                        : Text(
                            vo.descriptionText!,
                            maxLines: 1,
                            overflow: TextOverflow.ellipsis,
                            style: TextStyle(
                                fontSize: 11.w, color: Color(0xFF999999)),
                          ),
                    SizedBox(height: 4.w),
                    Row(
                      children: [
                        Container(
                          decoration: BoxDecoration(
                              border: Border.all(
                                  width: 0.5.w, color: Color(0x80FF8000)),
                              borderRadius: BorderRadius.only(
                                  topLeft: Radius.circular(8.w),
                                  topRight: Radius.circular(8.w),
                                  bottomRight: Radius.circular(8.w))),
                          padding: EdgeInsets.symmetric(
                              vertical: 1.w, horizontal: 4.w),
                          child: Text(vo.rebate!,
                              style: TextStyle(
                                  color: Color(0xFFFF8000),
                                  fontSize: 11.w,
                                  height: 1.3)),
                        ),
                        SizedBox(
                          width: 2.w,
                        ),
                        Expanded(
                          child: Container(
                            alignment: Alignment.centerLeft,
                            child: ListView.builder(
                              itemCount:
                                  vo.tags!.length >= 1 ? 1 : vo.tags!.length,
                              scrollDirection: Axis.horizontal,
                              itemBuilder: (BuildContext context, int index) {
                                return Container(
                                  margin: EdgeInsets.only(left: 4.w),
                                  decoration: BoxDecoration(
                                      color: Color(0xFFF5F5F5),
                                      borderRadius: BorderRadius.all(
                                          Radius.circular(2.w))),
                                  padding:
                                      EdgeInsets.symmetric(horizontal: 2.w),
                                  child: Text('${vo.tags![index]}',
                                      style: TextStyle(
                                          color: Color(0xFFB0B0B0),
                                          fontSize: 11.w,
                                          fontWeight: FontWeight.w400)),
                                );
                              },
                            ),
                            height: 16.w,
                          ),
                        ),
                      ],
                    ),
                    SizedBox(height: 4.w),
                    Text(
                      vo.special!,
                      style: TextStyle(
                          color: Color(0xFFFF0400),
                          fontSize: 13.w,
                          fontWeight: FontWeight.w400),
                    ),
                  ],
                ),
              )
            ],
          ),
        ));
  }
}

class PriceListWidget extends StatelessWidget {
  final EdgeInsetsGeometry padding;
  final bool shrinkWrap;
  final ScrollPhysics? physics;
  final void Function(ProductItemEntity, int)? onItemTap;
  final bool scroll;
  final List<ProductItemEntity>? list;
  final bool isHaoJia;
  const PriceListWidget(
      {Key? key,
      this.padding = const EdgeInsets.all(0),
      this.scroll = false,
      this.shrinkWrap = false,
      this.isHaoJia = false,
      this.physics,
      this.onItemTap,
      this.list})
      : super(key: key);
  @override
  Widget build(BuildContext context) {
    return ListView.builder(
      shrinkWrap: shrinkWrap,
      physics: physics,
      padding: padding,
      itemCount: list!.length,
      itemBuilder: (BuildContext context, int index) {
        ProductItemEntity itemVo = list![index];
        return _buildPriceItem(
          index: index,
          scroll: scroll,
          itemEntity: itemVo,
          onTap: () {
            if (onItemTap != null) {
              onItemTap!(itemVo, index);
            }
          },
        );
      },
    );
  }

  Widget _buildPriceItem(
      {required int index,
      required bool scroll,
      required ProductItemEntity itemEntity,
      void Function()? onTap}) {
    return InkWell(
      child: Container(
        decoration: BoxDecoration(
            color: Colors.white,
            borderRadius: BorderRadius.all(Radius.circular(8.w))),
        height: 120.w,
        margin: EdgeInsets.only(
            top: (index > 0)
                ? 8.w
                : isHaoJia
                    ? 0
                    : 8.w),
        child: Row(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            _buildLeftImage(
              imageUrl: itemEntity.itemPic,
              source: itemEntity.platform ?? '未知',
              scroll: scroll,
            ),
            _buildItemDetails(vo: itemEntity),
          ],
        ),
      ),
      onTap: onTap,
    );
  }

  ///
  /// 好价商品条目的标题等信息
  ///
  Widget _buildItemDetails({required ProductItemEntity vo}) {
    String? descriptionText = vo.descriptionText;
    if (descriptionText != null && descriptionText.isNotEmpty) {
      descriptionText = descriptionText.replaceAll('\n', '');
      descriptionText = descriptionText.replaceAll('\r', '');
    }
    return Expanded(
        child: Container(
      margin: EdgeInsets.only(top: 8.w, bottom: 10.w, right: 8.w),
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          Text(
            '${vo.itemTitle}',
            maxLines: 1,
            overflow: TextOverflow.ellipsis,
            style: TextStyle(
                fontSize: 14.w,
                color: Color(0xFF333333),
                height: 1.5,
                fontWeight: FontWeight.w600),
          ),
          SizedBox(
            height: 4,
          ),
          Text(
            '$descriptionText',
            maxLines: 1,
            overflow: TextOverflow.ellipsis,
            style: TextStyle(fontSize: 11.w, color: Color(0xFF999999)),
          ),
          Spacer(),
          Text(
            vo.special ?? '',
            style: TextStyle(
                fontSize: 13.w,
                color: Color(0xFFFF0400),
                fontWeight: FontWeight.w400),
            maxLines: 1,
            overflow: TextOverflow.ellipsis,
          ),
          SizedBox(
            height: 9.w,
          ),
          Container(
            child: Row(
              mainAxisAlignment: MainAxisAlignment.center,
              children: [
                Container(
                  decoration: BoxDecoration(
                      border:
                          Border.all(width: 0.5.w, color: Color(0x80FF8000)),
                      borderRadius: BorderRadius.only(
                          topLeft: Radius.circular(8.w),
                          topRight: Radius.circular(8.w),
                          bottomRight: Radius.circular(8.w))),
                  padding: EdgeInsets.symmetric(vertical: 1.w, horizontal: 4.w),
                  child: Text('${vo.rebate}',
                      style: TextStyle(
                        color: Color(0xFFFF8000),
                        fontSize: 11.w,
                        fontWeight: FontWeight.w500,
                        height: 1.3,
                      )),
                ),
                SizedBox(
                  width: 2.w,
                ),
                Expanded(
                  child: Container(
                    alignment: Alignment.centerLeft,
                    child: ListView.builder(
                      itemCount: vo.tags!.length >= 1 ? 1 : vo.tags!.length,
                      scrollDirection: Axis.horizontal,
                      itemBuilder: (BuildContext context, int index) {
                        return Container(
                          margin: EdgeInsets.only(left: 4.w),
                          decoration: BoxDecoration(
                              color: Color(0xFFF5F5F5),
                              borderRadius:
                                  BorderRadius.all(Radius.circular(2.w))),
                          padding: EdgeInsets.symmetric(horizontal: 4.w),
                          alignment: Alignment.center,
                          child: Text('${vo.tags![index]}',
                              style: TextStyle(
                                  color: Color(0xFF666666),
                                  fontSize: 11.w,
                                  fontWeight: FontWeight.w400)),
                        );
                      },
                    ),
                    height: 16.w,
                  ),
                ),
                Text(
                  '${vo.releaseTime}',
                  style: TextStyle(
                      color: Color(0xFFB0B0B0),
                      fontSize: 11.w,
                      fontWeight: FontWeight.w400),
                ),
              ],
            ),
          )
        ],
      ),
    ));
  }

  ///
  /// 商品图片
  /// [imageUrl]          商品图片
  /// [source]            商品来源，如淘宝、京东、拼多多等
  /// [scroll]            是否滚动
  ///
  Widget _buildLeftImage(
      {required String? imageUrl, required String source, required bool scroll}) {
    return Container(
      width: 104.w,
      height: 104.w,
      margin: EdgeInsets.all(8.w),
      child: Stack(
        alignment: Alignment.topRight,
        children: [
          scroll
              ? Image.asset('assets/images/placeholder_image.png',
                  package: 'price_module', width: 104.w, height: 104.w)
              : ClipRRect(
                  child: CachedNetworkImage(
                    imageUrl: imageUrl!,
                    placeholder: (BuildContext c, String s) {
                      return Image.asset(
                        'assets/images/placeholder_image.png',
                        package: 'price_module',
                      );
                    },
                    width: 104.w,
                    height: 104.w,
                    filterQuality: FilterQuality.low,
                  ),
                  borderRadius: BorderRadius.all(Radius.circular(4.w))),
          Container(
            decoration: BoxDecoration(
                color: Color(0xCCFFFFFF),
                border: Border.all(width: 0.5.w, color: Color(0xFFE9E9E9)),
                borderRadius: BorderRadius.all(Radius.circular(4.w))),
            margin: EdgeInsets.only(top: 4.w, right: 4.w),
            padding: EdgeInsets.symmetric(horizontal: 3.w),
            child: Text(source,
                style: TextStyle(color: Color(0xFF666666), fontSize: 10.w)),
          )
        ],
      ),
    );
  }
}
